#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""A Page component is defined that can read its content from the database
and display it
"""
from __future__ import with_statement

from nagare import presentation
from nagare.var import Var

from nagare.examples.wiki.wikidata import PageData   # The database entities

# ---------------------------------------------------------------------------

class Page(object):
    """A wiki page
    
    Its title is unique. It will be also the WikiWord others pages will use
    to link to this page.
    
    ``title`` is used to fetch the data from the database
    """
    def __init__(self, title):
        """Initialization
        
        In:
          - ``title`` -- the WikiWord title of the page
        """
        self.title = title

@presentation.render_for(Page)
def render(self, h, *args):
    """Display the raw content of the page
    """
    page = PageData.get_by(pagename=self.title) # Fetch all the data from the database

    # Display the content in a ``<pre>`` tag
    with h.div:
        h << h.pre(page.data)

    return h.root

# ---------------------------------------------------------------------------

app = lambda: Page(u'FrontPage') # The factory creates the ``FrontPage`` page



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""A Photo component is defined that can read its image, thumbnail and
title data from the database and display them
"""
from __future__ import with_statement

from nagare import presentation, component

from gallerydata import *

# ---------------------------------------------------------------------------

class Photo(object):
    """A Photo
    
    Its ``id`` is unique and is used to fetch the data from the database
    """
    def __init__(self, id):
        self.id = id

    @property
    def title(self):
        """Return the title of the photo"""
        return PhotoData.get(self.id).title

    def img(self):
        """Return the image data"""
        return str(PhotoData.get(self.id).img)

    def thumbnail(self):
        """Return the thumbnail data"""
        return str(PhotoData.get(self.id).thumbnail)

@presentation.render_for(Photo)
def render(self, h, *args):
    """The default view of a Photo displays it as a thumbnail with its title
    and its size"""

    with h.div:
        h << h.img(width='200').action(self.thumbnail)
        h << h.br
        h << self.title
        h << h.i(' (%d octets)' % len(self.img()))

    return h.root

# ---------------------------------------------------------------------------

# Display the first photo
app = lambda: Photo(1)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.examples.demo import Examples
from nagare.examples.gallery import gallery1, gallery2, gallery3, gallery4, gallery5, gallery6, gallery7

examples = (
            ('gallery1', ((gallery1.__doc__, gallery1.app),)),
            ('gallery2', ((gallery2.__doc__, gallery2.app),)),
            ('gallery3', ((gallery3.__doc__, gallery3.app),)),
            ('gallery4', ((gallery4.__doc__, gallery4.app),)),
            ('gallery5', ((gallery5.__doc__, gallery5.app),)),
            ('gallery6', ((gallery6.__doc__, gallery6.app),)),
            ('gallery7', ((gallery7.__doc__, gallery7.app),)),
           )

# TRAC url to display a module code from the SVN repository
SVN_EXAMPLES_URL = 'http://www.nagare.org/trac/browser/trunk/nagare/examples/nagare/examples/gallery/%s.py'

app = lambda: Examples('Gallery tutorial', SVN_EXAMPLES_URL, examples)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.examples.demo import Examples
from nagare.examples.wiki import wiki1, wiki2, wiki3, wiki4, wiki5, wiki6, wiki7, wiki8, wiki9

examples = (
            ('wiki1', ((wiki1.__doc__, wiki1.app),)),
            ('wiki2', ((wiki2.__doc__, wiki2.app),)),
            ('wiki3', ((wiki3.__doc__, wiki3.app),)),
            ('wiki4', ((wiki4.__doc__, wiki4.app),)),
            ('wiki5', ((wiki5.__doc__, wiki5.app),)),
            ('wiki6', ((wiki6.__doc__, wiki6.app),)),
            ('wiki7', ((wiki7.__doc__, wiki7.app),)),
            ('wiki8', ((wiki8.__doc__, wiki8.app),)),
            ('wiki9', ((wiki9.__doc__, wiki9.app),)),
           )

# TRAC url to display a module code from the Mercurial repository
HG_EXAMPLES_URL = 'http://www.nagare.org/trac/browser/examples/nagare/examples/wiki/%s.py'

app = lambda: Examples('Wiki tutorial', HG_EXAMPLES_URL, examples)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Examples to demonstrate the separation between UI components
and a logic component implemented as a ``component.Task``
"""

from __future__ import with_statement

import operator

from nagare import component, presentation, util

# ---------------------------------------------------------------------------

class TicTacToe:
    """The TicTacToe board
    """
    def __init__(self):
        """Initialization
        """
        # TicTacToe board as a linear list of cell
        self._board = [0]*9

    def played(self, player, played):
        """A cell was played
        
        In:
          - ``player`` -- id of the player that has played (1 or 2)
          - ``played`` -- the cell
        """
        self._board[played] = player

    def is_won(self):
        """Review all the cells played to check is the game is won
        
        Return:
          - a boolean: is the game ended and won ?
        """
        # Game won if 3 consecutive cells were played by the same player
        for (c1, c2, c3) in ((0, 1, 2), (3, 4, 5), (6, 7, 8), (0, 3, 6), (1, 4, 7), (2, 5, 8), (0, 4, 8), (2, 4, 6)):
            if self._board[c1] and (self._board[c1] == self._board[c2]) and (self._board[c2] == self._board[c3]):
                return True

        return False

    def is_ended(self):
        """Review all the cells playes to see if the game is ended (no more
        free cells)
        
        Return:
          - a boolean: is the game ended ?
        """
        return all(self._board)

@presentation.render_for(TicTacToe)
def render(self, h, comp, *args):
    """Render the board in a table of 3*3 cells. The free cells are clickable
    and answer their indice
    
    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component
      
    Return:
      - the view
    """
    h.head.css_url('tictactoe.css')

    with h.table(class_='tictactoe'):
        i = 0
        for row in zip(self._board[::3], self._board[1::3], self._board[2::3]):
            with h.tr:
                for x in row:
                    with h.td:
                        if x == 0:
                            h << h.a('-').action(lambda i=i: comp.answer(i))
                        else:
                            h << ('X' if x==1 else 'O')

                        i += 1
    return h.root

# ---------------------------------------------------------------------------

class Task(component.Task):
    """This ``component.Task`` uses continuations to implement the TicTacToe
    logic in pure linear Python code
    """
    
    def go(self, comp):
        while True:
            # 1. Create the board
            board = TicTacToe()

            # 2. Ask the names of the players
            players = (comp.call(util.Ask('Player #1')), comp.call(util.Ask('Player #2')))
    
            player = 1
            # 3. Play the game until a player wins or there are no more free cells
            while not board.is_won() and not board.is_ended():
                player = (player+1) & 1 # Toggle the player
    
                # Display the board and get the clicked cell
                played = comp.call(component.Component(board))
                
                # Register the clicked cell
                board.played(player+1, played)
    
            if board.is_won():
                msg =  'Player %s WON !' %  players[player]
            else:
                msg = 'Nobody WON !'
    
            # 4. Display the end message
            comp.call(util.Confirm(msg))

examples = ("Using a component.Task() to implement the game logic", Task)


# ---------------------------------------------------------------------------

# This example:
#
# - shows how to render a component with an asynchronous renderer
# - shows that the code of a component doesn't need to be modified to
#   become asynchronous

class App(object):
    def __init__(self):
        self.inner = component.Component(Task())

@presentation.render_for(App)
def render(self, h, *args):
    return self.inner.render(h.AsyncRenderer())

examples += ('Automatic use of asynchronous requests/updates', App)

# ---------------------------------------------------------------------------

class Double:
    def __init__(self):
        # Incremented each time the page is fully re-generated
        self.nb_display = 0

        # 2 different components of the same type
        self.left = component.Component(Task())
        self.right = component.Component(Task())

@presentation.render_for(Double)
def render(self, h, binding, *args):
    self.nb_display += 1

    with h.div:
        h << h.div('Full page generations: ',  self.nb_display)

        with h.div(style='float: left'):
            h << h.div('Synchronous', style='text-align: center; margin-top: 10px')

            # The ``left`` component is rendered with a standard HTML renderer
            h << self.left

        with h.div(style='float: right'):
            h << h.div('Asynchronous', style='text-align: center; margin-top: 10px')

            # The ``right`` component is rendered with an asynchronous HTML renderer
            h << h.td(self.right.render(h.AsyncRenderer()))

    return h.root

examples += ('Mixing 2 synchronous / asynchronous components', Double)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

try:
    from PIL import Image, ImageFilter, ImageDraw
except ImportError:
    import Image, ImageFilter, ImageDraw

import StringIO

def dropShadow(image, offset=(5,5), background=0xffffff, shadow=0x444444,
                border=8, iterations=3):
  """
  Add a gaussian blur drop shadow to an image.

  In:
    - ``image`` -- The image to overlay on top of the shadow.
    - ``offset`` -- Offset of the shadow from the image as an (x,y) tuple.  Can be
      positive or negative.
    - ``background`` -- Background colour behind the image.
    - ``shadow`` -- Shadow colour (darkness).
    - ``border`` -- Width of the border around the image.  This must be wide
      enough to account for the blurring of the shadow.
    - ``iterations`` -- Number of times to apply the filter.  More iterations
      produce a more blurred shadow, but increase processing time.

  The author of this function is Kevin Schluff:
    http://code.activestate.com/recipes/474116/
  """
  # Create the backdrop image -- a box in the background colour with a
  # shadow on it.
  totalWidth = image.size[0] + abs(offset[0]) + 2*border
  totalHeight = image.size[1] + abs(offset[1]) + 2*border
  back = Image.new(image.mode, (totalWidth, totalHeight), background)

  # Place the shadow, taking into account the offset from the image
  shadowLeft = border + max(offset[0], 0)
  shadowTop = border + max(offset[1], 0)
  back.paste(shadow, [shadowLeft, shadowTop, shadowLeft + image.size[0], shadowTop + image.size[1]])

  # Apply the filter to blur the edges of the shadow.  Since a small kernel
  # is used, the filter must be applied repeatedly to get a decent blur.
  n = 0
  while n < iterations:
      back = back.filter(ImageFilter.BLUR)
      n += 1

  # Paste the input image onto the shadow backdrop
  imageLeft = border - min(offset[0], 0)
  imageTop = border - min(offset[1], 0)
  back.paste(image, (imageLeft, imageTop))

  return back


def outline(image, border=5, color=0x000000):
    (w, h) = image.size

    back = Image.new(image.mode, (w+border*2, h+border*2), 0xffffff)
    back.paste(image, (border, border))

    draw = ImageDraw.Draw(back)
    draw.rectangle((0, 0, w+border*2-1, h+border*2-1), outline=color)
    #w+border*2, h+border*2))

    return back


def thumbnail(image):
  thumb = Image.open(StringIO.StringIO(image))
  format = thumb.format

  if thumb.mode == 'P':
      thumb.thumbnail((200,200))
  else:
      thumb.thumbnail((200,200), Image.ANTIALIAS)
      thumb = dropShadow(outline(thumb, border=8, color=0x666666), shadow=0x666666)

  i = StringIO.StringIO()
  thumb.save(i, format)
  return i.getvalue()

# ---------------------------------------------------------------------------

if __name__ == "__main__":
  import sys

  image = Image.open(sys.argv[1])
  image.thumbnail( (200,200), Image.ANTIALIAS)

  dropShadow(outline(image, border=8, color=0x666666), shadow=0x666666).show()
  #dropShadow(image, background=0xeeeeee, shadow=0x444444, offset=(0,5)).show()



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

#from nagare.database import session

from sqlalchemy import MetaData
from sqlalchemy.ext import declarative

__metadata__ = MetaData()
Entity = declarative.declarative_base(metadata=__metadata__)

# ---------------------------------------------------------------------------

def populate2():
    database.session.add(GalleryData(name=u'MyGallery'))



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from elixir import *
from sqlalchemy import MetaData

__metadata__ = MetaData()

class PhotoData(Entity):
    title = Field(Unicode(100))
    img = Field(BLOB)
    thumbnail = Field(BLOB)
    
    belongs_to('gallery', of_kind='GalleryData')

        
class GalleryData(Entity):
    name = Field(Unicode(40))
   
    has_many('photos', of_kind='PhotoData')

# ---------------------------------------------------------------------------

def populate1():
    import os

    gallery = GalleryData(name=u'MyGallery')
  
    f = file(os.path.join(os.path.dirname(__file__), 'dragon.jpg'))
    img = f.read()
    f.close()
        
    gallery.photos.append(PhotoData(title=u'Dragon', img=img, thumbnail=img))

def populate2():
    GalleryData(name=u'MyGallery')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

from nagare import presentation, component, comet, var, ajax

# -----------------------------------------------------------------------------

@ajax.javascript
def append_msg(msg):
    # Add a ``<li>`` to the ``<ul>`` list with id ``msgs``
    #
    # This function is translated to javascript and executed on the browser
    #
    # In:
    #   - ``msg`` -- message to add

    # Create the ``<li>`` element, filled with the message text
    li = document.createElement('li')
    li.appendChild(document.createTextNode(msg))
    li.setAttribute('class', 'msg_type_'+msg.substring(0, 1))

    msgs = document.getElementById('msgs')

    # The ``<li>`` becomes the first child of the list
    if not msgs.childNodes.length:
        msgs.appendChild(li)
    else:
        msgs.insertBefore(li, msgs.firstChild)

# -----------------------------------------------------------------------------

class User(object):
    """User interactions
    """
    def __init__(self):
        """Initialization
        """
        self.user = var.Var()   # The user name

    def logout(self, comp):
        """The user wants to quit the chat room

        In:
          - ``comp`` -- this component
        """
        comp.answer(('part', self.user()))
        self.user(None)

@presentation.render_for(User)
def render(self, h, comp, *args):
    msg = var.Var()

    if not self.user():
        # If the user is not connected, generate the login form
        # -----------------------------------------------------

        with h.form:
            h << "What's your name: " << h.input.action(self.user)
            h << h.input(type='submit', value='Enter the room').action(lambda: comp.answer(('join', self.user())) )
    else:
        # If the user is connected, generate the form to enter and send a message
        # -----------------------------------------------------------------------

        with h.div:
            h << h.a('Logout').action(lambda: self.logout(comp))

            with h.form:
                h << h.input(size=50).action(msg) << ' '
                h << h.input(type='submit', value='Send').action(lambda: comp.answer(('send', self.user(), msg())))

    return h.root

# -----------------------------------------------------------------------------

class Chat(object):
    """Chat room logic
    """
    def __init__(self, channel_id):
        """Initialization

        In:
          - ``channel_id`` -- identifier of the push channel to use
        """
        self.channel_id = channel_id

        self.interaction = component.Component(User())
        self.interaction.on_answer(self.actions)    # Dispatch on the answer

        # Create the push channel
        # ``append_msg.name`` is the name of the generated javascript function
        comet.channels.create(channel_id, append_msg.name)

    def actions(self, args):
        """Actions dispatcher

        In:
          - ``args`` -- ``args[0]`` is the action name, ``args[1:]`` its parameters
        """
        getattr(Chat, args[0])(self, *args[1:])

    def join(self, user):
        """A user enters the chat room

        In:
          - ``user`` -- the user name
        """
        comet.channels.send(self.channel_id, 'join: '+user)

    def part(self, user):
        """A user quits the chat room

        In:
          - ``user`` -- the user name
        """
        comet.channels.send(self.channel_id, 'part: '+user)

    def send(self, user, msg):
        """A user sends a message to the chat room

        In:
          - ``user`` -- the user name
          - ``msg`` -- the message
        """
        comet.channels.send(self.channel_id, 'msg from %s: %s' % (user, msg))

@presentation.render_for(Chat)
def render(self, h, *args):
    h.head.css('nagare_chat', '''
               #msgs {
                        list-style-type: none;
                        border: 1px dashed #f3f2f1;
                        padding: 5px;
               }
               #msgs li:nth-child(odd) { background-color: #f3f2f1; }
               .msg_type_j { color: red }
               .msg_type_p { color: blue }

               ''')

    # Inclusion of the translated ``append_msg`` function
    h.head << h.head.script(ajax.py2js(append_msg, h))

    with h.div:
        # Automatic inclusion of the javascript Comet functions
        h << component.Component(comet.channels[self.channel_id])

        # Asynchronous (Ajax) rendering of the user interaction form
        h << self.interaction.render(h.AsyncRenderer())

        # This list will be filled by the received messages
        h << h.ul(id='msgs')

    return h.root

# -----------------------------------------------------------------------------

def app():
    return Chat('nagare_chat')




#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

import random

from nagare import component, util, state

# ---------------------------------------------------------------------------

class Number(component.Task):
    """A little game to guess a number
    """
    def __init__(self, final_text):
        self.final_text = final_text
        
    def go(self, comp):
        """The game algorithm, using continuation for a pure linear Python code
        
        In:
          - ``comp`` -- this component
        """
        self.attempt = 1
        number = random.randint(1, 20)
        
        comp.call(util.Confirm('I choose a number between 1 and 20. Try to guess it'))
        
        while True:
            x = comp.call(util.Ask('Try #%d: ' % self.attempt))
            if not x.isdigit():
                continue
            
            x = int(x)
            
            if x > number:
                comp.call(util.Confirm('Choose a lower number'))
                
            if x < number:
                comp.call(util.Confirm('Choose a greater number'))
                
            if x == number:
                comp.call(util.Confirm(self.final_text % self.attempt))
                break

            self.attempt += 1

# ---------------------------------------------------------------------------

# This example shows that, by default, the objects are statefull

def example1():
    # By default, an object is statefull
    number = Number('''You found the secret number in %d tries.
Now use the 'back' button to cheat : go back to the first attempt and enter the correct number''')
    return number

examples = ('A statefull component', example1)

# ---------------------------------------------------------------------------

# This example shows how to create stateless objects

def example2():
    number = Number('''You found the secret number in %d tries.
Now use the 'back' button to cheat : you CAN'T''')
    
    # The object is made stateless
    number = state.stateless(number)
    
    return number

examples += ('A stateless component', example2)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Form, to append a new photo to a gallery, added"""

from __future__ import with_statement

from nagare import presentation, component, editor

from gallerydata import *

# ---------------------------------------------------------------------------

class Photo(object):
    def __init__(self, id):
        self.id = id

    @property
    def title(self):
        return PhotoData.get(self.id).title

    def img(self):
        return str(PhotoData.get(self.id).img)

    def thumbnail(self):
        return str(PhotoData.get(self.id).thumbnail)

@presentation.render_for(Photo)
def render(self, h, comp, *args):   
    img = h.img.action(self.img)
    return h.a(img).action(comp.answer)

@presentation.render_for(Photo, model='thumbnail')
def render(self, h, comp, *args):
    with h.div:
        h << h.img(width='200').action(self.thumbnail)
        h << h.br
        h << h.a(self.title).action(lambda: comp.answer(self))
        h << h.i(' (%d octets)' % len(self.img()))

    return h.root

# A PhotoCreator is defined, to download a new Photo
class PhotoCreator(editor.Editor):
    def __init__(self):
        """At this stage, we are using ``editor.Property`` but without
          validation functions"""
        self.title = editor.Property(None)
        self.img = editor.Property(None)

    def commit(self, comp):
        """After the submission of the form, the PhotoCreator answers the
           tuple (title of the photo, image data)"""
        comp.answer((self.title(), self.img.value.file.read()))

@presentation.render_for(PhotoCreator)
def render(self, h, comp, *args):
    """The default view of a PhotoCreator component is the upload form"""
    with h.form:
        with h.table(border=0):
            with h.tr:
                # The action on the "Title" field is to call ``self.title`` i.e
                # put the value of the field into the ``title`` property
                h << h.td('Title') << h.td(':') << h.td(h.input.action(self.title))

            with h.tr:
                # The action on the "Image" field is to call ``self.image`` i.e
                # put the value of the field into the ``image`` property
                h << h.td('Image') << h.td(':') << h.td(h.input(type='file').action(self.img))

            with h.tr:
                h << h.td << h.td
                with h.td:
                    # The action on the "Add" button is to call ``self.commit``, i.e
                    # answer the tuple (title of the photo, image data)
                    h << h.input(type='submit', value='Add').action(lambda: self.commit(comp))
                    
                    h << ' '
                    
                    # The action on the "Cancel" button is to call ``comp.answer``, i.e
                    # answer ``None``
                    h << h.input(type='submit', value='Cancel').action(comp.answer)

    return h.root

# ---------------------------------------------------------------------------

class Gallery(object):
    def __init__(self, name):
        self.name = name

    def add_photo(self, comp):
        """This method temporary replaces the Gallery component by a PhotoCreator
           component. Upon submission of a photo, the PhotoCreator answers with
           the title and the uploaded image"""
           
        # Change the Gallery by a PhotoCreator component
        r = comp.call(PhotoCreator())
        
        if r is not None:
            # If the user click on the the "Cancel" button, we receive ``None``
            # else we receive the title and the uploaded image
            (title, img) = r

            # Insert these data into the database
            gallery = GalleryData.get_by(name=self.name)
            gallery.photos.append(PhotoData(title=title, img=img, thumbnail=img))

@presentation.render_for(Gallery)
def render(self, h, comp, *args):
    with h.div:
        h << h.h1('Gallery: ', self.name)
        
        # In the default Gallery view, add a link to add a new photo
        h << h.a('Add photo').action(lambda: self.add_photo(comp))

        h << h.br

        with h.ul:
            for p in GalleryData.get_by(name=self.name).photos:
                photo = component.Component(Photo(p.id))
                photo.on_answer(comp.call)

                h << h.li(photo.render(h, model='thumbnail'))

    return h.root

# ---------------------------------------------------------------------------

app = lambda: Gallery(u'MyGallery')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

import operator
from nagare import component, presentation, util

class TicTacToe:
    def __init__(self):
        self._board = [0]*9

    def played(self, player, played):
        self._board[played] = player

    def is_won(self):
        for (c1, c2, c3) in ((0, 1, 2), (3, 4, 5), (6, 7, 8),
                             (0, 3, 6), (1, 4, 7), (2, 5, 8), (0, 4, 8), (2, 4, 6)):
            if self._board[c1] and (self._board[c1] == self._board[c2] == self._board[c3]):
                return True

        return False

    def is_ended(self):
        return all(self._board)

@presentation.render_for(TicTacToe)
def render(self, h, comp, *args):
    h.head.css_url('tictactoe.css')

    with h.table(class_='tictactoe'):
        i = 0
        for row in zip(self._board[::3], self._board[1::3], self._board[2::3]):
            with h.tr:
                for x in row:
                    with h.td:
                        if x == 0:
                            h << h.a('-').action(lambda i=i: comp.answer(i))
                        else:
                            h << ('X' if x==1 else 'O')

                        i += 1
    return h.root

class Task(component.Task):
    def go(self, comp):
        while True:
            board = TicTacToe()

            players = (comp.call(util.Ask('Player #1')),
                       comp.call(util.Ask('Player #2')))

            player = 1
            while not board.is_won() and not board.is_ended():
                player = (player+1) & 1

                played = comp.call(component.Component(board))
                board.played(player+1, played)

            if board.is_won():
                msg =  'Player %s WON !' %  players[player]
            else:
                msg = 'Nobody WON !'

            comp.call(util.Confirm(msg))

# -----------------------------------------------------------------------------

hl_lines = (
    range(12, 72),
    (
        (4,),
        'Definition of a Plain Old Python Object',
        range(4, 21)
    ),

    (
        (22,),
        '<p>Definition of its default HTML view</p>'
        '<p>Parameters are:'
        '<ol>'
        '<li><code>self</code>: the <code>TicTacToe</code> object</li>'
        '<li><code>h</code>: a HTML renderer</li>'
        '<li><code>comp</code>: the component wrapping the <code>TicTacToe</code> object</li>'
        '</ol>',
        range(22, 39)
    ),

    (
        (33, 52),
        '<p>Use of the <code>call()/answer()</code> mechanism: the task calls a <code>TictacToe</code> component</p>'
        '<p>When an empty cell is clicked, the cell index is sent back to the task '
        'as <code>played</code></p>',
        (33, 52)
    ),

    (
        (40,),
        'A <code>component.Task</code> uses continuations to implement the '
        'TicTacToe logic in pure linear Python code',
        range(40, 61)
    ),

    (
        (45, 60),
        'Usage of the Nagare components <code>util.Ask</code> '
        'and <code>util.Confirm</code>',
        (45, 46, 60)

    )
)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""A RPN calculator"""

import operator
from nagare import presentation

class Calculator:
    """A RPN calculator object with methods to manage the stack
    """
    def __init__(self):
        """Initialisation"""
        self.display = ''   # Number displayed
        self.stack = []     # RPN stack

    def digit(self, digit):
        """A new digit is entered

        In:
          - ``digit``
        """
        self.display += str(digit)

    def enter(self):
        """The ``enter`` is pressed, push the number displayed onto the stack
        """
        if self.display:
            self.stack.append(int(self.display))
            self.display = ''

    def operand(self, op):
        """Do a calculus. Push the result onto the stack.

        In:
          - ``op`` -- 2 operands function
        """
        self.enter()

        if len(self.stack) >= 2:
            v = self.stack.pop()
            self.stack[-1] = op(self.stack[-1], v)

    def drop(self):
        """Clear the display or drop the top of the stack"""
        if self.display:
            self.display = ''
            return

        if not self.stack:
            return

        self.stack.pop()

    def get_last(self):
        """Return the number displayed or the top of the stack

        Return:
          - the number
        """
        if self.display:
            return self.display

        if self.stack:
            return str(self.stack[-1])

        return '0'

@presentation.render_for(Calculator)
def render(self, h, *args):
    """Rendering of the RPN calculator

    In:
      - ``h`` -- the renderer

    Return:
      - the view
    """
    h.head << h.head.css_url('calculator.css')

    with h.div(class_='calculator'):
        h << h.div(self.get_last(), class_='calculator_display')

        with h.table:
            with h.tr:
                h << h.td(colspan=3)
                h << h.td(h.a('C').action(self.drop))
                h << h.td(h.a(u'\N{DIVISION SIGN}').action(lambda: self.operand(operator.div)))

            with h.tr:
                h << h.td(h.a(7).action(lambda: self.digit(7)))
                h << h.td(h.a(8).action(lambda: self.digit(8)))
                h << h.td(h.a(9).action(lambda: self.digit(9)))
                h << h.td
                h << h.td(h.a(u'\N{MULTIPLICATION X}').action(lambda: self.operand(operator.mul)))

            with h.tr:
                h << h.td(h.a(4).action(lambda: self.digit(4)))
                h << h.td(h.a(5).action(lambda: self.digit(5)))
                h << h.td(h.a(6).action(lambda: self.digit(6)))
                h << h.td
                h << h.td(h.a(u'\N{MINUS SIGN}').action(lambda: self.operand(operator.sub)))

            with h.tr:
                h << h.td(h.a(1).action(lambda: self.digit(1)))
                h << h.td(h.a(2).action(lambda: self.digit(2)))
                h << h.td(h.a(3).action(lambda: self.digit(3)))
                h << h.td
                h << h.td(h.a(u'\N{PLUS SIGN}').action(lambda: self.operand(operator.add)))

            with h.tr:
                h << h.td
                h << h.td(h.a(0).action(lambda: self.digit(0)))
                h << h.td(colspan=2)
                h << h.td(h.a(u'\N{WHITE RIGHT-POINTING POINTER}').action(self.enter))

    return h.root

examples = ('RPN calculator', Calculator)




#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Basic HTTP authentication of the users added.

First, log as the editor 'john / john'.
Then try as the administrator 'admin / admin'.
"""

from __future__ import with_statement

import re
import docutils.core

from nagare import component, presentation, var, security, wsgi

from wikidata import PageData

# ---------------------------------------------------------------------------

class Login:
    pass

@presentation.render_for(Login)
def render(self, h, *args):
    user = security.get_user()
    
    if not user:
        return h.i('not logged')
    
    return ('Welcome ', h.b(user.id))

# ---------------------------------------------------------------------------

wikiwords = re.compile(r'\b([A-Z]\w+[A-Z]+\w+)')

class Page(object):
    def __init__(self, title):
        self.title = title

    def edit(self, comp):
        content = comp.call(PageEditor(self))

        if content is not None:
            security.check_permissions('wiki.editor', self)            
            page = PageData.get_by(pagename=self.title)
            page.data = content

@presentation.render_for(Page)
def render(self, h, comp, *args):
    page = PageData.get_by(pagename=self.title)

    content = docutils.core.publish_parts(page.data, writer_name='html')['html_body']
    content = wikiwords.sub(r'<wiki>\1</wiki>', content)
    html = h.parse_htmlstring(content, fragment=True)[0]

    for node in html.getiterator():
        if node.tag == 'wiki':
            a = h.a(node.text, href='page/'+node.text).action(lambda title=unicode(node.text): comp.answer(title))
            node.replace(a)
    
    return (html, h.a('Edit this page', href='page/'+self.title).action(lambda: self.edit(comp)))

@presentation.render_for(Page, model='meta')
def render(self, h, comp, *args):
    return ('Viewing ', h.b(self.title), h.br, h.br,
            'You can return to the ', h.a('FrontPage', href='page/FrontPage').action(lambda: comp.answer(u'FrontPage')))

# ---------------------------------------------------------------------------

class PageEditor(object):
    def __init__(self, page):
        self.page = page

@presentation.render_for(PageEditor)
def render(self, h, comp, *args):
    content = var.Var()

    page = PageData.get_by(pagename=self.page.title)

    with h.form:
        with h.textarea(rows='10', cols='40').action(content):
            h << page.data
        h << h.br
        h << h.input(type='submit', value='Save').action(lambda: comp.answer(content()))
        h << ' '
        h << h.input(type='submit', value='Cancel').action(comp.answer)

    return h.root

@presentation.render_for(PageEditor, model='meta')
def render(self, h, *args):
    return ('Editing ', h.b(self.page.title))

# ---------------------------------------------------------------------------

class Wiki(object):
    def __init__(self):
        self.login = component.Component(Login())        
        self.content = component.Component(None)
        self.content.on_answer(self.goto)

        self.goto(u'FrontPage')

    def goto(self, title):
        page = PageData.get_by(pagename=title)
        if page is None:
            PageData(pagename=title, data='')

        self.content.becomes(Page(title))

@presentation.render_for(Wiki)
def render(self, h, comp, *args):
    h.head.css('main_css', '''
    .document:first-letter { font-size:2em }
    .meta { float:right; width: 10em; border: 1px dashed gray;padding: 1em; margin: 1em; }
    .login { font-size:0.75em; }    
    ''')

    with h.div(class_='login'):
        h << self.login

    with h.div(class_='meta'):
        h << self.content.render(h, model='meta')
        
    h << self.content << h.hr

    h << 'View the ' << h.a('complete list of pages', href='all').action(lambda: self.goto(comp.call(self, model='all')))
    
    return h.root

@presentation.render_for(Wiki, model='all')
@security.permissions('wiki.admin')
def render(self, h, comp, *args):
    with h.ul:
        for page in PageData.query.order_by(PageData.pagename):
            with h.li:
                h << h.a(page.pagename, href='page/'+page.pagename).action(lambda title=page.pagename: comp.answer(title))

    return h.root

# ---------------------------------------------------------------------------

@presentation.init_for(Wiki, "(len(url) == 2) and (url[0] == 'page')")
def init(self, url, *args):
    title = url[1]
    
    page = PageData.get_by(pagename=title)
    if page is None:
        raise presentation.HTTPNotFound()

    self.goto(title)
    
@presentation.init_for(Wiki, "len(url) and (url[0] == 'all')")
def init(self, url, comp, *args):
    component.call_wrapper(lambda: self.goto(comp.call(self, model='all')))

# ---------------------------------------------------------------------------

from peak.rules import when
from nagare.security import common

# We define 3 classes:
#
#  - the ``User`` class defines the attributes of the users
#  - the ``Authentication`` class defines how the users are identified /
#    authenticated
#  - the ``HardcodedRules`` class defines the mapping between the users and
#    the permissions

class User(common.User):
    pass


# We use the basic HTTP authentication to identify and authenticate the
# users
from nagare.security import basic_auth

class Authentication(basic_auth.Authentication):
    def get_password(self, username):
        # For this example, the passwords are the same than the users name
        return username
    
    # When a user is identified and authenticated, then create a ``User`` object
    def _create_user(self, username):
        return None if username is None else User(username)


class HardcodedRules(common.Rules):
    # A logged user wants to access an object protected by the 'wiki.editor'
    # permission
    @when(common.Rules.has_permission, "user and (perm == 'wiki.editor')")
    def _(self, user, perm, subject):
        # Grant access only to 'john' and 'admin' 
        return user.id in ('john', 'admin')

    # A logged user wants to access an object protected by the 'wiki.admin'
    # permission
    @when(common.Rules.has_permission, "user and (perm == 'wiki.admin')")
    def _(self, user, perm, subject):
        # Grant access only to 'admin' 
        return user.id == 'admin'
    

# A security manager is often a mixin of an authentication manager and
# a security rules manager
class SecurityManager(Authentication, HardcodedRules):
    pass

# ---------------------------------------------------------------------------

# We create a new WSGIApp class that has our security manager as its `security`
# attribute
class WSGIApp(wsgi.WSGIApp):
    def __init__(self, app_factory):
        super(WSGIApp, self).__init__(app_factory)
        self.security = SecurityManager(realm='My wiki')
        
app = WSGIApp(lambda: component.Component(Wiki()))



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""A Gallery component is defined that manage a set of Photos
"""

from __future__ import with_statement

from nagare import presentation, component

from gallerydata import *

# ---------------------------------------------------------------------------

class Photo(object):
    def __init__(self, id):
        self.id = id

    @property
    def title(self):
        return PhotoData.get(self.id).title

    def img(self):
        return str(PhotoData.get(self.id).img)

    def thumbnail(self):
        return str(PhotoData.get(self.id).thumbnail)

@presentation.render_for(Photo, model='thumbnail')
def render(self, h, *args):
    """View renamed to ``thumbnail``"""
    with h.div:
        h << h.img(width='200').action(self.thumbnail)
        h << h.br
        h << self.title
        h << h.i(' (%d octets)' % len(self.img()))

    return h.root


class Gallery(object):
    def __init__(self, name):
        """A gallery has a name"""
        self.name = name

@presentation.render_for(Gallery)
def render(self, h, *args):
    """The default view of a Gallery displays the list of the Photos"""
    
    with h.div:
        h << h.h1('Gallery: ', self.name)
        h << h.br

        with h.ul:
            # Use the database relation to get all the photos of this
            # gallery
            for p in GalleryData.get_by(name=self.name).photos:
                # Create a Photo object with the id of the photo
                # then make it a component to display ir
                photo = component.Component(Photo(p.id))

                # Render the ``thumbnail`` view of the Photo component
                h << h.li(photo.render(h, model='thumbnail'))

    return h.root

# ---------------------------------------------------------------------------

app = lambda: Gallery(u'MyGallery')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Adding a second view to the Page and PageEditor component, displaying
meta data about the current page and action. 
"""

from __future__ import with_statement

import re
import docutils.core

from nagare import component, presentation, var

from wikidata import PageData

# ---------------------------------------------------------------------------

wikiwords = re.compile(r'\b([A-Z]\w+[A-Z]+\w+)')

class Page(object):
    def __init__(self, title):
        self.title = title

    def edit(self, comp):
        content = comp.call(PageEditor(self))

        if content is not None:
            page = PageData.get_by(pagename=self.title)
            page.data = content

@presentation.render_for(Page)
def render(self, h, comp, *args):
    page = PageData.get_by(pagename=self.title)

    content = docutils.core.publish_parts(page.data, writer_name='html')['html_body']
    content = wikiwords.sub(r'<wiki>\1</wiki>', content)
    html = h.parse_htmlstring(content, fragment=True)[0]

    for node in html.getiterator():
        if node.tag == 'wiki':
            a = h.a(node.text).action(lambda title=unicode(node.text): comp.answer(title))
            node.replace(a)

    return (html, h.a('Edit this page').action(lambda: self.edit(comp)))

@presentation.render_for(Page, model='meta')
def render(self, h, comp, *args):
    """Second view, called ``meta``, for the Page component
    
    Display:
    
      - the current action : 'Viewing'
      - the current page title
      - a link to return to the 'FrontPage' page
    """
    return ('Viewing ', h.b(self.title), h.br, h.br,
            'You can return to the ', h.a('FrontPage').action(lambda: comp.becomes(Page(u'FrontPage'))))

# ---------------------------------------------------------------------------

class PageEditor(object):
    def __init__(self, page):
        self.page = page

@presentation.render_for(PageEditor)
def render(self, h, comp, *args):
    content = var.Var()

    page = PageData.get_by(pagename=self.page.title)

    with h.form:
        with h.textarea(rows='10', cols='40').action(content):
            h << page.data
        h << h.br
        h << h.input(type='submit', value='Save').action(lambda: comp.answer(content()))
        h << ' '
        h << h.input(type='submit', value='Cancel').action(comp.answer)

    return h.root

@presentation.render_for(PageEditor, model='meta')
def render(self, h, *args):
    """Second view, called ``meta``, for the PageEditor component
    
    Display:
    
      - the current action : 'Editing'
      - the title of the current edited page
    """    
    return ('Editing ', h.b(self.page.title))

# ---------------------------------------------------------------------------

class Wiki(object):
    def __init__(self):
        self.content = component.Component(None)
        self.content.on_answer(self.goto)

        self.goto(u'FrontPage')

    def goto(self, title):
        page = PageData.get_by(pagename=title)
        if page is None:
            PageData(pagename=title, data='')

        self.content.becomes(Page(title))

@presentation.render_for(Wiki)
def render(self, h, *args):
    h.head.css('main_css', '''
    .document:first-letter { font-size:2em }
    .meta { float:right; width: 10em; border: 1px dashed gray;padding: 1em; margin: 1em; }
    ''')

    return (
             # The following div display the 'meta' view of the current
             # ``self.content`` component. It can be a Page component or
             # a PageEditor component. We don't care because both has a
             # ``meta`` view.
             h.div(self.content.render(h, model='meta'), class_='meta'),
             self.content,
            )

# ---------------------------------------------------------------------------

app = Wiki



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""The default view for a Page will now format its ReStructuredText
content in HTML and activate the WikiWord. NOTE: at this stage, clicking
on a WikiWord will raise an error.
"""

from __future__ import with_statement

import re               # To detect the WikiWords
import docutils.core    # To translate the ReST in HTML

from nagare import presentation, var

from wikidata import PageData

# ---------------------------------------------------------------------------

# WikiWords regular expression
wikiwords = re.compile(r'\b([A-Z]\w+[A-Z]+\w+)')

class Page(object):
    def __init__(self, title):
        self.title = title

    def edit(self, comp):
        content = comp.call(PageEditor(self))
        
        if content is not None:
            page = PageData.get_by(pagename=self.title)
            page.data = content

@presentation.render_for(Page)
def render(self, h, comp, *args):
    page = PageData.get_by(pagename=self.title)

    # Translate the content in ``page.data`` to HTML
    content = docutils.core.publish_parts(page.data, writer_name='html')['html_body']
    
    # For each WikiWords found, create a pseudo tag '<wiki>WikiWord</<wiki>'
    content = wikiwords.sub(r'<wiki>\1</wiki>', content)
    
    # Parse the HTML into a elements tree
    html = h.parse_htmlstring(content, fragment=True)[0]

    # Found the '<wiki>' elements into the tree and transform them into active links
    for node in html.getiterator():
        if node.tag == 'wiki':
            # Clicking on a WikiWord will answer the WikiWord unicode string
            a = h.a(node.text).action(lambda title=unicode(node.text): comp.answer(title))
            # Replace the node
            node.replace(a)

    return (html, h.a('Edit this page').action(lambda: self.edit(comp)))

# ---------------------------------------------------------------------------

class PageEditor(object):
    def __init__(self, page):
        self.page = page
        
@presentation.render_for(PageEditor)
def render(self, h, comp, *args):
    content = var.Var()
    
    page = PageData.get_by(pagename=self.page.title)
    
    with h.form:
        with h.textarea(rows='10', cols='40').action(content):
            h << page.data
        h << h.br
        h << h.input(type='submit', value='Save').action(lambda: comp.answer(content()))
        h << ' '
        h << h.input(type='submit', value='Cancel').action(comp.answer)
                                                             
    return h.root

# ---------------------------------------------------------------------------

app = lambda: Page(u'FrontPage')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""A PageEditor component is added to edit the content of a page
"""

from __future__ import with_statement

from nagare import presentation, var

from wikidata import PageData

# ---------------------------------------------------------------------------

class Page(object):
    def __init__(self, title):
        self.title = title

    def edit(self, comp):
        """Edit the content of this page:
        
           1. a page editor is created
           2. the page editor temporary replace the page in the component graph
           3. it will return (and restore the page) with the new content of the
              page

        In:
          - ``comp`` -- this component
        """
        # A ``PageEditor`` object is created for ``self``
        # It temporary replaces the current component (the page displayed)
        content = comp.call(PageEditor(self))
        
        # ``content`` is the new content for the page
        # or ``None`` if the user canceled the modification
        if content is not None:
            # Retrieve the database object
            page = PageData.get_by(pagename=self.title)
            # Change the content
            page.data = content
        
@presentation.render_for(Page)
def render(self, h, comp, *args):
    page = PageData.get_by(pagename=self.title)
    
    with h.div:    
        h << h.pre(page.data)

        # This link is added to every page displayed, to be able to edit it
        h << h.a('Edit this page').action(lambda: self.edit(comp))

    return h.root

# ---------------------------------------------------------------------------

class PageEditor(object):
    """An object dedicated to the edition of a page content
    """
    def __init__(self, page):
        """Initialization
        
        In:
          - ``page`` -- the page to edit
        """
        self.page = page
        
@presentation.render_for(PageEditor)
def render(self, h, comp, *args):
    content = var.Var() # Local functional variable that will keep the new page content
    
    # Retrieve the database object
    page = PageData.get_by(pagename=self.page.title)
    
    with h.form:
        with h.textarea(rows='10', cols='40').action(content): # Calling ``content`` with a value set its
            h << page.data
        h << h.br
        # Clicking on 'Save', will answer the new content
        h << h.input(type='submit', value='Save').action(lambda: comp.answer(content()))
        h << ' '
        # Clicking on 'Cancel', will answer ``None``
        h << h.input(type='submit', value='Cancel').action(comp.answer)
                                                             
    return h.root

# ---------------------------------------------------------------------------

app = lambda: Page(u'FrontPage')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Examples to demonstrate the embedding, binding, replacing and
call/answer of components"""
 
from nagare import component
from nagare import presentation

class ColorChooser:
    """A simple object"""
    pass

@presentation.render_for(ColorChooser)
def render(self, h, comp, *args):
    """This view answered the color clicked
    
    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component
      
    Return:
      - the view
    """
    return (
            h.a('X', style='background-color: #4B96FF').action(lambda: comp.answer('#4B96FF')),
            h.a('X', style='background-color: #FF8FDF').action(lambda: comp.answer('#FF8FDF')),
            h.a('X', style='background-color: #9EFF5D').action(lambda: comp.answer('#9EFF5D')),
            h.a('X', style='background-color: #FFFFFF').action(lambda: comp.answer('#FFFFFF'))
           )
    # Could also be written as:
    #return [h.a('X', style='background-color:' + color).action(lambda c=color: comp.answer(c))
    #        for color in ('#4B96FF', '#FF8FDF', '#9EFF5D', '#FFFFFF')]

"""
@presentation.render_for(ColorChooser, model='meta')
def render(self, h, *args):
    return h.h1(self.id)
"""

class Text:
    """A colorised text"""
    def __init__(self, text='', color='#ffffff'):
        """Initialization
        
        In:
          - ``text``
          - ``color`` -- background color of the text
        """
        self.color = color
        self.text = text

    def set_color(self, color):
        """Change the background color
        
        In:
          - ``color`` -- new background color of the text
        """
        self.color = color

@presentation.render_for(Text)
def render(self, h, *args):
    """This view renders the text on a colorised background
    
    In:
      - ``h`` -- the renderer
      
    Return:
      - the view
    """
    return h.span(self.text, style='background-color: '+self.color)

"""
@presentation.render_for(Text, model='meta')
def render(self, h, *args):
    return h.h2('Text')
"""

# ---------------------------------------------------------------------------

# This example:
#
# - render a component

def example1():
    return Text('Hello world !', 'lightgreen')
    
examples = ('A simple colorised text', example1)

# -------------------------------------------------------------------------------------------------------

# This example:
#
# - embed and bind components

class App1:
    """This component:
    
      - embeds 2 components: a text component and a color chooser
      - bind the answer of the color chooser (the selected color) to the
        ``set_color()`` method of the text component
    """
    def __init__(self):
        self.text = component.Component(Text('Hello world !'))

        self.chooser = component.Component(ColorChooser())
        self.chooser.on_answer(self.text().set_color)   # Binding

@presentation.render_for(App1)
def render(self, h, *args):
    """This view renders the 2 embedded components
    
    In:
      - ``h`` -- the renderer
      
    Return:
      - the view
    """    
    return (self.chooser, h.br, self.text)

@presentation.render_for(App1, model='meta')
def render(self, h, *args):
    return h.h2('Yeah !')

examples += ('Embedding and binding components', App1)

# -------------------------------------------------------------------------------------------------------

# This example:
#
# - embed and bind components

class App2:
    """This component implement a simple workflow:
    
      1. display the text component
      2. replace the text component to display the color chooser
      3. after a color is selected, display the text component with this
         color
    """    
    def __init__(self):
        self.text = component.Component(Text('Hello world !'))

    def change_text_color(self, comp):
        """Replace the text component with the color chooser by calling it.
        Then, as the color chooser answers with the selected color, set the color
        text with it
        
        In:
          - ``comp`` -- the component to replace by the color chooser
        """
        # Temporary replace the component by the color chooser
        color = comp.call(ColorChooser())
        
        # Set the background color of the text with the selected color
        self.text().set_color(color)

@presentation.render_for(App2)
def render(self, h, comp, *args):
    """This view renders the 2 embedded components
    
    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component
      
    Return:
      - the view
    """    
    return (
             h.a('Change the color').action(lambda: self.change_text_color(comp)),
             h.br,
             self.text,
            )

examples += ("Call / answer of a component", App2)

# -------------------------------------------------------------------------------------------------------

# This example:
#
# - illustrate that the same object can be displayed several time with
#   different views

class App3:
    def __init__(self):
        self.app = component.Component(App1())

@presentation.render_for(App3)
def render(self, h, *args):
    """This view renders twice the same component but with different views
    (the default view and the ``meta`` view)
    
    In:
      - ``h`` -- the renderer
      
    Return:
      - the view
    """        
    return h.table(h.tr(
                        # The component is rendered with the default view
                        h.td(self.app, align='center'),
                        
                        # The component is rendered with the ``meta`` view
                        h.td(self.app.render(h, model='meta'), align='center')
                       ))

examples += ('Twice the same component with 2 different views', App3)

# -------------------------------------------------------------------------------------------------------

# This example:
#
# - demonstrate that each component follow its own workflow

class Double:
    def __init__(self):
        self.left  = component.Component(App1())
        self.right = component.Component(App2())

@presentation.render_for(Double)
def render(self, h, *args):
    return h.table(h.tr(h.td(self.left, align='center'),
                        h.td(self.right, align='center')))

examples += ('2 independante components', Double)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Adding significative URLs"""

from __future__ import with_statement

from nagare import presentation, component, editor, validator

from gallerydata import *
import thumb

# ---------------------------------------------------------------------------

class Photo(object):
    def __init__(self, id):
        self.id = id

    @property
    def title(self):
        return PhotoData.get(self.id).title

    def img(self):
        return str(PhotoData.get(self.id).img)

    def thumbnail(self):
        return str(PhotoData.get(self.id).thumbnail)

@presentation.render_for(Photo)
def render(self, h, comp, *args):
    img = h.img.action(self.img)
    return h.a(img).action(comp.answer)

@presentation.render_for(Photo, model='thumbnail')
def render(self, h, comp, *args):
    with h.div:
        h << h.img.action(self.thumbnail)
        h << h.br
        h << h.a(self.title).action(lambda: comp.answer(self))
        h << h.i(' (%d octets)' % len(self.img()))

    return h.root


class PhotoCreator(editor.Editor):
    def __init__(self):
        self.title = editor.Property(None)
        self.img = editor.Property(None)

        self.title.validate(lambda t: validator.to_string(t).not_empty().to_string())
        self.img.validate(self.validate_img)

    def validate_img(self, img):
        if isinstance(img, basestring):
            raise ValueError, 'Image not provided'
        return img.file.read()

    def commit(self, comp):
        if self.is_validated(('title', 'img')):
            comp.answer((self.title(), self.img.value))

@presentation.render_for(PhotoCreator)
def render(self, h, comp, *args):
    with h.form:
        with h.table(border=0):
            with h.tr:
                h << h.td('Title') << h.td(':') << h.td(h.input.action(self.title).error(self.title.error))

            with h.tr:
                h << h.td('Image') << h.td(':') << h.td(h.input(type='file').action(self.img).error(self.img.error))

            with h.tr:
                h << h.td() << h.td()
                with h.td:
                    h << h.input(type='submit', value='Add', id='submitbutton').action(lambda: self.commit(comp))
                    h << ' '
                    h << h.input(type='submit', value='Cancel', id='submitbutton').action(comp.answer)

    return h.root

# ---------------------------------------------------------------------------

class Gallery(object):
    def __init__(self, name):
        self.name = name
        if GalleryData.get_by(name=name) is None:
            GalleryData(name=name)

    def add_photo(self, comp):
        r = comp.call(PhotoCreator())
        if r is not None:
            (title, img) = r

            gallery = GalleryData.get_by(name=self.name)
            gallery.photos.append(PhotoData(title=title, img=img, thumbnail=thumb.thumbnail(img)))

@presentation.render_for(Gallery)
def render(self, h, comp, *args):
    h.head.css('gallery', '''
    ul.photo_list {
        list-style: none;
    }

    ul.photo_list li {
        display: block;
        float: left;
        border: 1px dashed gray;
        padding: 1em;
        margin: 1em;
    }
    ''')

    with h.div:
        h << h.h1('Gallery: ', self.name)
        h << h.a('Add photo', style='float: right').action(lambda: self.add_photo(comp))
        h << h.br

        with h.ul(class_='photo_list'):
            for p in GalleryData.get_by(name=self.name).photos:

                # The url for a photo is its title
                photo = component.Component(Photo(p.id), url=p.title)

                photo.on_answer(comp.call)

                h << h.li(photo.render(h, model='thumbnail'))

    return h.root

# From a URL received, set the components
@presentation.init_for(Gallery, "len(url) == 1")
def init(self, url, comp, *args):
    # The URL received is the name of the photo
    photo_data = PhotoData.get_by(title=url[0])
    if not photo_data:
        # A photo with this name doesn't exist
        raise presentation.HTTPNotFound()

    # Get the photo data and make a ``Photo`` component
    photo = component.Component(Photo(photo_data.id))

    # Temporary change the Gallery (the ``comp``) with the photo
    component.call_wrapper(lambda: comp.call(photo))

# ---------------------------------------------------------------------------

app = lambda: Gallery(u'MyGallery2')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Adding significative URLs"""

from __future__ import with_statement

from sqlalchemy import Column, Unicode, BLOB, ForeignKey, Integer
from sqlalchemy.orm import relation

from nagare import presentation, component, editor, validator
from nagare.database import session, query

from gallerydeclarative import Entity
import thumb

class Photo(Entity):
    __tablename__ = 'photo_base'

    title = Column(Unicode(100), primary_key=True)
    img = Column(BLOB)
    thumbnail = Column(BLOB)
    gallery_id = Column(Integer, ForeignKey('gallery_data.name'))

    def __init__(self, title, img, thumbnail):
        self.title = title
        self.img = img
        self.thumbnail = thumbnail

@presentation.render_for(Photo)
def render(self, h, comp, *args):
    img = h.img.action(lambda h: str(self.img))
    return h.a(img).action(comp.answer)

@presentation.render_for(Photo, model='thumbnail')
def render(self, h, comp, *args):
    with h.div:
        h << h.img.action(lambda h: str(self.thumbnail))
        h << h.br
        h << h.a(self.title).action(lambda: comp.answer(self))
        h << h.i(' (%d octets)' % len(self.img))

    return h.root


class PhotoCreator(editor.Editor):
    def __init__(self):
        self.title = editor.Property(None)
        self.img = editor.Property(None)

        self.title.validate(lambda t: validator.to_string(t).not_empty().to_string())
        self.img.validate(self.validate_img)

    def validate_img(self, img):
        if isinstance(img, basestring):
            raise ValueError, 'Image not provided'
        return img.file.read()

    def commit(self, comp):
        if self.is_validated(('title', 'img')):
            comp.answer((self.title(), self.img.value))

@presentation.render_for(PhotoCreator)
def render(self, h, comp, *args):
    with h.form:
        with h.table(border=0):
            with h.tr:
                h << h.td('Title') << h.td(':') << h.td(h.input.action(self.title).error(self.title.error))

            with h.tr:
                h << h.td('Image') << h.td(':') << h.td(h.input(type='file').action(self.img).error(self.img.error))

            with h.tr:
                h << h.td() << h.td()
                with h.td:
                    h << h.input(type='submit', value='Add', id='submitbutton').action(lambda: self.commit(comp))
                    h << ' '
                    h << h.input(type='submit', value='Cancel', id='submitbutton').action(comp.answer)

    return h.root

# ---------------------------------------------------------------------------

class Gallery(Entity):
    __tablename__ = 'gallery_data'

    name = Column(Unicode(40), primary_key=True)
    photos = relation('Photo', backref='gallery')

    def __init__(self, name):
        self.name = name

    def add_photo(self, comp):
        r = comp.call(PhotoCreator())
        if r is not None:
            (title, img) = r

            photo = Photo(title, img, thumbnail=thumb.thumbnail(img))
            self.photos.append(photo)
            database.session.add(photo)

@presentation.render_for(Gallery)
def render(self, h, comp, *args):
    h.head.css('gallery', '''
    ul.photo_list {
        list-style: none;
    }

    ul.photo_list li {
        display: block;
        float: left;
        border: 1px dashed gray;
        padding: 1em;
        margin: 1em;
    }
    ''')

    with h.div:
        h << h.h1('Gallery: ', self.name)
        h << h.a('Add photo', style='float: right').action(lambda: self.add_photo(comp))
        h << h.br

        with h.ul(class_='photo_list'):
            for photo in self.photos:
                # The url for a photo is its title
                photo = component.Component(photo, url=photo.title)
                photo.on_answer(comp.call)

                h << h.li(photo.render(h, model='thumbnail'))

    return h.root

# From a URL received, set the components
@presentation.init_for(Gallery, "len(url) == 1")
def init(self, url, comp, *args):
    # The URL received is the name of the photo
    photo = query(Photo).get(url[0])
    if not photo:
        # A photo with this name doesn't exist
        raise presentation.HTTPNotFound()

    # Temporary change the Gallery (the ``comp``) with the photo
    component.call_wrapper(lambda: comp.call(component.Component(photo)))

# ---------------------------------------------------------------------------

app = lambda: query(Gallery).get(u'MyGallery')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

from nagare import component, presentation, var

# Modules where the examples are located
import counter
import calculator
import color
import tictactoe
import form
import number

# ---------------------------------------------------------------------------

class Menu(object):
    """This component displays a menu
    """
    def __init__(self, labels, selected=0):
        """Initialization

        In:
          - ``labels`` -- list of menu entries (title, url)
          - ``selected`` -- selected label
        """
        self.labels = labels
        self._selected = selected   # The currently selected label

@presentation.render_for(Menu)
def render(self, h, comp, *args):
    with h.ul(class_='menu'):
        for (i, (label, href)) in enumerate(self.labels):
            # When clicked, an entry answers with its index
            a = h.a(label, href=href).action(lambda i=i: comp.answer(i))
            if i == self._selected:
                a(class_='selected')

            h << h.li(a)

    return h.root

# ---------------------------------------------------------------------------

class Examples(object):
    """Display the UI to browse the examples
    """
    def __init__(self, title, repository_path, examples):
        """Initialization

        In:
          - ``title`` -- title to display
          - ``repository_path`` -- url template to display a module code
          - ``examples`` -- list of examples
        """
        self.title = title
        self.repository_path = repository_path
        self.examples = examples

        # The ``example`` component is displayed on the right side of the
        # screen. It will be replaced each time an example is selected
        self.example = component.Component(None)

        # The ``tree_view`` component is displayed on the left side of the
        # screen. It will answered the selected example
        self.level1_menu = component.Component(None)

        # Each time an example is selected in the menu, call the
        # ``change_example()`` method
        self.level1_menu.on_answer(self.change_example)

        self.level2_menu = component.Component(None)
        self.level2_menu.on_answer(self.change_step)

        # Select the first example
        self.change_example(0)

    def change_example(self, i):
        """A new example is selected

        In:
          - ``i`` -- index of the example to select
        """
        self.selected_examples = i;
        self.module_name = self.examples[i][0]

        self.level1_menu.becomes(Menu([(label.capitalize(), label) for (label, _) in self.examples], i))

        self.change_step(0)

    def change_step(self, i):
        """A new step of the current example is selected

        In:
          - ``i`` -- step to select
        """
        selected_examples = self.examples[self.selected_examples][1]
        self.description = selected_examples[i][0]

        self.example.becomes(selected_examples[i][1](), url='%s/%d' % (self.module_name, i+1))

        if len(selected_examples) != 1:
            labels = [('%s %d' % (self.module_name, j), j) for j in range(1, len(selected_examples)+1)]
            self.level2_menu.becomes(Menu(labels, i), url=self.module_name)
        else:
            self.level2_menu.becomes(None)

@presentation.render_for(Examples)
def render(self, h, comp, *args):
    """Display the full UI
    """
    # Add some CSS into the page header
    h.head.css_url('/static/nagare/application.css')
    h.head.css_url('demo.css')

    h.head << h.head.title(self.title)

    with h.div(id='body'):
        h << h.a(h.img(src='/static/nagare/img/logo.png'), id='logo', href='http://www.nagare.org/', title='Nagare home')

        with h.div(id='content'):
            h << h.div(self.title, id='title')

            h << h.div(self.level1_menu, id='mainnav', class_='horizontal')

            with h.div(id='main'):
                if self.level2_menu():
                    h << h.div(self.level2_menu, id='steps')

                with h.div(id='example', class_='example_right' if self.level2_menu() else 'example_full' ):
                    h << h.div(self.description, class_='note')

                    with h.div(id='source'):
                        h << h.a('View the source', href=self.repository_path % self.module_name)

                    h << self.example

    return h.root

# ---------------------------------------------------------------------

@presentation.init_for(Examples, "len(url) == 1")
def init(self, url, comp, http_method, request):
    example_name = url[0]

    try:
        example = [x[0] for x in self.examples].index(example_name)
    except ValueError:
        raise presentation.HTTPNotFound()

    self.change_example(example)

@presentation.init_for(Examples, "len(url) >= 2 and http_method == 'GET'")
def init(self, url, comp, http_method, request):
    comp.init(url[:1], http_method, request)

    step = url[1]

    try:
        step = int(step)
    except ValueError:
        raise presentation.HTTPNotFound()

    if (step <= 0) or (step > len(self.examples[self.selected_examples][1])):
        raise presentation.HTTPNotFound()

    self.change_step(step-1)

    if len(url) > 2:
        self.example.init(url[2:], http_method, request)

# ---------------------------------------------------------------------

def module_name(name):
    """From a full Python module name, return only the module name

    In:
      - ``name`` -- Python module name

    Return:
      - module name
    """
    # Keep only the last part of the path
    return name.rsplit('.', 1)[1]


# Modules where to located the ``examples`` list
modules = (counter, calculator, color, tictactoe, number, form)

# Create the examples list
examples = [(module_name(module.__name__), zip(module.examples[::2], module.examples[1::2])) for module in modules]

# TRAC url to display a module code from the Mercurial repository
HG_EXAMPLES_URL = 'http://www.nagare.org/trac/browser/examples/nagare/examples/%s.py'

app = lambda: Examples('Demo', HG_EXAMPLES_URL, examples)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Basic examples of components, multiple views and synchronous/asynchronous
callbacks
"""

from __future__ import with_statement

from nagare import component, presentation

examples = ()

# ---------------------------------------------------------------------

# This example:
#
# - shows how to add multiples web views on a Python object
# - shows how to associate method actions to HTML elements

class Counter1:
    """A simple counter with ``increase`` and ``decrease`` logics
    """
    def __init__(self, start=0):
        self.v = start

    def increase(self):
        self.v += 1

    def decrease(self):
        self.v -= 1

@presentation.render_for(Counter1)
def render(self, h, comp, *args):
    """Same view but written in "imperative HTML"

    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component

    Return:
      - the view of the referenced object
    """
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        h << h.div(self.v)

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', title='decrease').action(self.decrease)

        with h.span:
            h << h.a('=', title='freeze').action(lambda: comp.becomes(self, model='freezed'))

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', title='increase').action(self.increase)

    return h.root

@presentation.render_for(Counter1, model='freezed')
def render(self, h, *args):
    """An other view, displaying only the counter value, without any actions
    possible

    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component

    Return:
      - the view of the referenced object
    """
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        h << h.div(self.v)

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', class_='disabled')

        with h.span:
            h << h.a('=', class_='disabled')

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', class_='disabled')

    return h.root

examples += ('Multiple views and methods callbacks', Counter1)

'''
@presentation.render_for(Counter1)
def render(self, h, comp, *args):
    """This view is written in "functional HTML"

    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component

    Return:
      - the view of the referenced object
    """
    return (
             h.div('Value: ', self.v),
             h.a('++').action(self.increase),
             '|',
             h.a('--').action(self.decrease),
             h.hr,
             h.a('Freeze it !').action(lambda: comp.becomes(self, model='freeze'))
           )
'''

# ---------------------------------------------------------------------------

# This example:
#
# - shows how to add multiples web views on a Python object
# - shows how to associate lambda actions to HTML elements

from nagare.var import Var

class Counter2:
    """The value is kept into a functional variable. Easier to work with,
    into lambda expressions.
    """
    def __init__(self, start=0):
        self.v = Var(start)

@presentation.render_for(Counter2)
def render(self, h, comp, *args):
    """View with lambdas as actions

    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component

    Return:
      - the view of the referenced object
    """
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        h << h.div(self.v)

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', title='decrease').action(lambda: self.v(self.v()-1))

        with h.span:
            h << h.a('=', title='freeze').action(lambda: comp.becomes(self, model='freezed'))

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', title='increase').action(lambda: self.v(self.v()+1))

    return h.root

@presentation.render_for(Counter2, model='freezed')
def render(self, h, *args):
    """An other view, displaying only the counter value, without any actions
    possible

    In:
      - ``h`` -- the renderer
      - ``comp`` -- the component

    Return:
      - the view of the referenced object
    """
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        h << h.div(self.v)

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', class_='disabled')

        with h.span:
            h << h.a('=', class_='disabled')

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', class_='disabled')

    return h.root

examples += ('Multiple views and lambdas callbacks', Counter2)

# ---------------------------------------------------------------------

# This example:
#
# - shows how to render a component with an asynchronous renderer
# - shows that the code of a component doesn't need to be modified to
#   become asynchronous

class App:
    def __init__(self):
        # Incremented each time the page is fully re-generated
        self.nb_display = 0

        # 2 different components of the same type
        self.counter1 = component.Component(Counter1())
        self.counter2 = component.Component(Counter1())

@presentation.render_for(App)
def render(self, h, *args):
    self.nb_display += 1

    with h.div:
        h << h.div('Full page generation: ', self.nb_display)

        with h.div(style='float: left; margin-top: 10px'):
            h << h.div('Synchronous', style='text-align: center')

            # The ``counter1`` component is rendered with a standard HTML renderer
            h << self.counter1

        with h.div(style='float: right; margin-top: 10px'):
            h << h.div('Asynchronous', style='text-align: center')

            # The ``counter2`` component is rendered with an asynchronous HTML renderer
            h << self.counter2.render(h.AsyncRenderer())

    return h.root

examples += ('Automatic use of asynchronous requests/updates', App)

# ---------------------------------------------------------------------

# This example:
#
# - shows how to explicitly use a ``ajax.Update`` callback to change a HTML
#   element asynchronously (using its id)

from nagare import ajax

class Counter4:
    def __init__(self, start=0):
        self.v = start

    def increase(self):
        self.v += 1

    def decrease(self):
        self.v -= 1

@presentation.render_for(Counter4)
def render(self, h, *args):
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        h << h.div(self.v, id='value')

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', title='decrease').action(ajax.Update(lambda h: str(self.v), self.decrease, 'value'))

        with h.span:
            h << h.a('=', class_='disabled')

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', title='increase').action(ajax.Update(lambda h: str(self.v), self.increase, 'value'))

    return h.root

examples += ('Asynchronous update of a HTML element', Counter4)

# ---------------------------------------------------------------------

# This example:
#
# - shows how to explicitly use a ``ajax.Update`` callback to change a HTML
#   element asynchronously

"""
@presentation.render_for(Counter4, model='without_id')
def render(self, h, *args):
    # HTML element to update
    div = h.div(0)

    return (
            div,

            # ``ajax.Update`` object that:
            #
            #   - calls ``self.increase`` as action
            #   - calls ``lambda h: str(self.v)`` to render the view
            #   - updates the given HTML element
            h.a('++').action(ajax.Update(lambda h: str(self.v), self.increase, div)),

            ' | ',

            # ``ajax.Update`` object that:
            #
            #   - calls ``self.decrease`` as action
            #   - calls ``lambda h: str(self.v)`` to render the view
            #   - updates the given HTML element
            h.a('--').action(ajax.Update(lambda h: str(self.v), self.decrease, div)),
           )
"""

@presentation.render_for(Counter4, model='without_id')
def render(self, h, *args):
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        div = h.div(self.v)

        h << div

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', title='decrease').action(ajax.Update(lambda h: str(self.v), self.decrease, div))

        with h.span:
            h << h.a('=', class_='disabled')

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', title='increase').action(ajax.Update(lambda h: str(self.v), self.increase, div))

    return h.root

class App2:
    def __init__(self):
        self.counter = component.Component(Counter4(), model='without_id')

@presentation.render_for(App2)
def render(self, h, *args):
    return self.counter.render(h)

examples += ('Asynchronous update of a HTML element', App2)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

from nagare import presentation, editor
from lxml.html import clean

class Html:
    def __init__(self):
        self.content = ''

@presentation.render_for(Html)
def default_view(self, h, comp, *args):
    with h.div:
        if self.content:
            h << h.parse_htmlstring(self.content)

        h << h.a('Edit HTML').action(lambda: comp.call(HTMLEditor(self)))

    return h.root


def to_valid_html(value, msg="Not a valid html fragment"):
    try:
        return clean.Cleaner().clean_html(value)
    except:
        raise ValueError(msg)


class HTMLEditor(editor.Editor):
    fields = ('content', )

    def __init__(self, target):
        super(HTMLEditor, self).__init__(target, self.fields)
        self.content.validate(to_valid_html)

    def commit(self, comp):
        if super(HTMLEditor, self).commit(self.fields):
            comp.answer()

@presentation.render_for(HTMLEditor)
def default_view(self, h, comp, *args):
    with h.form:
        h << h.label('HTML source code')
        h << h.textarea(self.content, rows=15, cols=30).action(self.content).error(self.content.error)
        h << h.input(type='submit', value='Save').action(lambda: self.commit(comp))
        h << ' '
        h << h.input(type='submit', value='Cancel').action(comp.answer)

    return h.root

# -----------------------------------------------------------------------------

hl_lines = (
    range(12, 59),
    (
        (4,),
        '<p>A <code>Html</code> widget is defined</p>'
        '<p>Its default view displays its HTML content</p>',
        range(4, 17)
    ),

    (
        (14, 42),
        '<p>Use of the <code>call()/answer()</code> mechanism: a <code>Html</code> widget '
        'calls an <code>HTMLEditor</code></p>'
        '<p>When the <code>Save</code> button is clicked, the <code>content</code> is changed, '
        'the editor answers and the <code>Html</code> widget is rendered back</p>',
        [14, 33, 34, 35, 42]
    ),

    (
        (14, 44),
        '<p>Use of the <code>call()/answer()</code> mechanism: a <code>Html</code> widget '
        'calls an <code>HTMLEditor</code></p>'
        '<p>When the <code>Cancel</code> button is clicked the editor answers and the '
        '<code>Html</code> widget is rendered back',
        [14, 44]
    ),

    (
        (19, 31, 41),
        '<p><code>to_valid_html()</code> is a dedicated form field validator</p>'
        '<p>A validation method is associated to a Property object through its '
        '<code>validate()</code> method</p>',
        range(19, 24)+[31, 41]
    ),

    (
        (26,),
        '<p>Definition of a Nagare Editor</p>'
        '<p>Its default view is a form</p>'
        '<p><code>target</code> parameter is the edited <code>Html</code> widget</p>',
        range(26, 47)
    )
)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

import random

from nagare import component, presentation, util

class Board(object):
    """The Jewels board
    """
    NB_IMGS = 3

    def __init__(self, cols, rows):
        """Initialization

        In:
          - ``cols`` -- board width
          - ``rows`` -- board height
        """
        # Create an (cols+2)*(rows+2) array
        # (one empty row and col around the board)
        self.matrix = [[None]*(rows+2) for j in xrange(cols+2)]

        # Initialization each cells with a random jewel
        for i in xrange(1, rows+1):
            for j in xrange(1, cols+1):
                self.matrix[j][i] = random.randint(1, self.NB_IMGS)

        self.score = 0

    def is_finish(self):
        """Test if the game is finished

        Return:
          - a boolean
        """
        r = False

        # Test if two adjacent cells in a row have the same jewel
        for row in self.matrix:
            r |= any(a==b for (a, b) in zip(row, row[1:]) if a)

        # Test if two adjacent cells in a column have the same jewel
        for col in zip(*self.matrix):
            r |= any(a==b for (a, b) in zip(col, col[1:]) if a)

        return not r

    def flood(self, i, j, jewel):
        """Empty all the connected cells with the same jewel

        In:
          - ``i``, ``j`` -- the current cell to test
          - ``jewel`` -- the jewel to test

        Return:
          - number of connected cells
        """
        if self.matrix[j][i] != jewel:
            # The current cell has not this jewel
            # Stop the recursion
            return 0

        # Empty the current cell
        self.matrix[j][i] = None

        # Check all the adjacent cells
        return 1 + self.flood(i-1, j, jewel) + self.flood(i+1, j, jewel) + self.flood(i, j-1, jewel) + self.flood(i, j+1, jewel)

    def collapse(self):
        """Collapse the empty cells
        """
        self.matrix = [sorted(l, key=bool)[1:]+[None] for l in self.matrix]

        self.matrix.sort(key=any, reverse=True)
        self.matrix.insert(0, self.matrix.pop())

    def played(self, i, j):
        """Play a selected cell

        In:
          - ``i``, ``j`` -- the selected cell
        """
        jewel = self.matrix[j][i]
        n = self.flood(i, j, jewel) # Empty the connected cells

        # Add to the score: ((nb of removed cells) - 2) ** 2
        if n == 1:
            self.matrix[j][i] = jewel
        else:
            self.score += (n-2)**2

        self.collapse()

@presentation.render_for(Board)
def render(self, h, comp, *args):
    with h.div(class_='jewels'):
        h << h.h1('Score: ', self.score)

        with h.table:
            for (i, row) in enumerate(zip(*self.matrix)[1:-1]):
                with h.tr:
                    for (j, jewel) in enumerate(row[1:-1]):
                        with h.td(class_='cell%d' % ((i+j) % 2)):
                            if jewel:
                                # Clicking on a cell with a jewel answers the tuple (i, j)
                                with h.a.action(lambda i=i+1, j=j+1: comp.answer((i, j))):
                                    h << h.img(src='jewel%d.gif' % jewel)

    return h.root

# -----------------------------------------------------------------------------

class Jewels(component.Task):
    """The Jewels game
    """
    def __init__(self, cols, rows):
        """Initialization

        In:
          - ``cols`` -- board width
          - ``rows`` -- board height
        """
        self.cols = cols
        self.rows = rows

    def go(self, comp):
        # Create the board
        board = Board(self.cols, self.rows)

        while not board.is_finish():
            # Wait for a click in a cell
            played = comp.call(component.Component(board))

            # Play it
            board.played(*played)

        comp.call(util.Confirm('No movements left, you scored %s.' % board.score))

# -----------------------------------------------------------------------------

class App(object):
    """Jewels game example
    """

    def __init__(self):
        """Initialization
        """
        # Create a jewels component with a default size
        self.jewels = component.Component(Jewels(15, 10))

@presentation.render_for(App)
def render(self, h, *args):
    h.head.css('jewels', '''
        body {
            background-color: #335577;
            font-family: Verdana;
        }''')

    h.head.css_url('jewels.css')
    h.head << h.head.title('Jewels')

    with h.div(class_='jewels'):
        h << self.jewels

    return h.root

# -----------------------------------------------------------------------------

app = App



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

import operator
from nagare import presentation

class Calculator:
    def __init__(self):
        self.display = ''
        self.stack = []

    def digit(self, digit):
        self.display += str(digit)

    def enter(self):
        if self.display:
            self.stack.append(int(self.display))
            self.display = ''

    def operand(self, op):
        self.enter()

        if len(self.stack) >= 2:
            v = self.stack.pop()
            self.stack[-1] = op(self.stack[-1], v)

    def drop(self):
        if self.display:
            self.display = ''
            return

        if not self.stack:
            return

        self.stack.pop()

    def get_last(self):
        if self.display:
            return self.display

        if self.stack:
            return str(self.stack[-1])

        return '0'

@presentation.render_for(Calculator)
def render(self, h, *args):
    h.head << h.head.css_url('calculator.css')

    with h.div(class_='calculator'):
        h << h.div(self.get_last(), class_='calculator_display')

        with h.table:
            with h.tr:
                h << h.td(colspan=3)
                h << h.td(h.a('C').action(self.drop))
                h << h.td(h.a(u'\N{DIVISION SIGN}').action(lambda: self.operand(operator.div)))

            with h.tr:
                h << h.td(h.a(7).action(lambda: self.digit(7)))
                h << h.td(h.a(8).action(lambda: self.digit(8)))
                h << h.td(h.a(9).action(lambda: self.digit(9)))
                h << h.td
                h << h.td(h.a(u'\N{MULTIPLICATION X}').action(lambda: self.operand(operator.mul)))

            with h.tr:
                h << h.td(h.a(4).action(lambda: self.digit(4)))
                h << h.td(h.a(5).action(lambda: self.digit(5)))
                h << h.td(h.a(6).action(lambda: self.digit(6)))
                h << h.td
                h << h.td(h.a(u'\N{MINUS SIGN}').action(lambda: self.operand(operator.sub)))

            with h.tr:
                h << h.td(h.a(1).action(lambda: self.digit(1)))
                h << h.td(h.a(2).action(lambda: self.digit(2)))
                h << h.td(h.a(3).action(lambda: self.digit(3)))
                h << h.td
                h << h.td(h.a(u'\N{PLUS SIGN}').action(lambda: self.operand(operator.add)))

            with h.tr:
                h << h.td
                h << h.td(h.a(0).action(lambda: self.digit(0)))
                h << h.td(colspan=2)
                h << h.td(h.a(u'\N{WHITE RIGHT-POINTING POINTER}').action(self.enter))

    return h.root

# -----------------------------------------------------------------------------

hl_lines = (
    range(12, 95),
    (
        (4,),
        'Definition of a Plain Old Calculator Python Object',
        range(4, 42)
    ),

    (
        (43,),
        '<p>Default view for a <code>Calculator</code></p>'
        '<p>Parameters are:'
        '<ol>'
        '<li><code>self</code>: the <code>Calculator</code> object</li>'
        '<li><code>h</code>: a HTML renderer</li>'
        '</ol>',
        range(43, 84)
    ),

    (
        (53,),
        'Direct association of the <code>drop</code> method to a link',
        (53,)
    ),

    (
        (54, 61, 68, 75),
        'Use of anonymous functions to call the <code>operand</code> method',
        (54, 61, 68, 75)
    ),

    (
        (57, 58, 59, 64, 65, 66, 71, 72, 73, 79),
        'Use of anonymous functions to call the <code>digit</code> method',
        (57, 58, 59, 64, 65, 66, 71, 72, 73, 79)
    ),

    (
        (81,),
        'Direct association of the <code>enter</code> method to link',
        (81,)
    )
)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

from nagare import component, presentation

class Counter:
    def __init__(self, start=0):
        self.v = start

    def increase(self):
        self.v += 1

    def decrease(self):
        self.v -= 1

@presentation.render_for(Counter)
def render(self, h, comp, *args):
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        h << h.div(self.v)

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', title='decrease').action(self.decrease)

        with h.span:
            h << h.a('=', title='freeze').action(lambda: comp.becomes(self, model='freezed'))

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', title='increase').action(self.increase)

    return h.root

@presentation.render_for(Counter, model='freezed')
def render(self, h, *args):
    h.head.css_url('counter.css')

    with h.div(class_='counter'):
        h << h.div(self.v)

        with h.span:
            h << h.a(u'\N{MINUS SIGN}', class_='disabled')

        with h.span:
            h << h.a('=', class_='disabled')

        with h.span:
            h << h.a(u'\N{PLUS SIGN}', class_='disabled')

    return h.root

# -----------------------------------------------------------------------------

hl_lines = (
    range(12, 60),
    (
        (13,),
        '<p>Default view for a <code>Counter</code> component</p>'
        '<p>Parameters are:'
        '<ol>'
        '<li><code>self</code>: the <code>Counter</code> object</li>'
        '<li><code>h</code>: a HTML renderer</li>'
        '<li><code>comp</code>: the component wrapping the '
        '<code>Counter</code> object</li></ol>',
        range(13, 30)
    ),

    (
        (21,),
        'Direct association of the <code>decrease</code> method to a link',
        (21,)
    ),

    (
        (27,),
        'Direct association of the <code>increase</code> method to a link',
        (27,)
    ),

    (
        (24,),
        '<p>Explicitly association of an anonymous function to  a link</p>'
        '<p>After a click an the link, the component will be rendered with its '
        '<code>freezed</code> view: the <code>becomes()</code> method of the '
        'component is used to change it by the same object but with a different '
        'view</p>',
        (24,)
    ),

    (
        (31,),
        '<p>Named view <code>freezed</code> for a <code>Counter</code> component',
        range(31, 48)
    ),
)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Dynamic generation of an image data

An action can be associated to an ``<img>` tag. This action must return
the image data.
"""

import urllib

from nagare import presentation

class Image(object):
    """A proxy image object to an image on the net"""
    def __init__(self, title, url):
        """Initialization
        
        In:
          - ``title`` -- title of the image
          - ``url`` -- url where to fetch the image data
        """
        self.title = title
        self.url = url
        
    def send_image(self):
        """Read the image data
        
        In:
          - ``h`` -- the current renderer

        Return:
          - the image data
        """
        return urllib.urlopen(self.url).read()

@presentation.render_for(Image)
def render(self, h, *args):
    # An action can be associated to a ``<img>`` tag
    return (h.h1(self.title), h.img.action(self.send_image))

app = lambda: Image('Hello world !', 'http://www.google.fr/intl/fr_fr/images/logo.gif')



# -*- coding: iso-8859-1 -*-

#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--


"""Examples of same reusable widgets"""

from __future__ import with_statement

import operator

from nagare import component, presentation, var

# -----------------------------------------------------------------------------------------

import time
import datetime

class Date(object):
    """A date widget to include into a ``<form>``
    """    
    months = (
                'january', 'february', 'mars', 'april',
                'may', 'june', 'july', 'august',
                'september', 'october', 'november', 'december'
             )

    def __init__(self, date):
        """Initialization
        
        In:
          - ``date`` -- a ``datetime`` object
        """
        self.day = var.Var(str(date.day))
        self.month = var.Var(str(date.month))
        self.year = var.Var(str(date.year))

    def to_datetime(self):
        """Return the data into a ``datetime`` object
        
        Return:
          - the datetime
        """
        return datetime.date(int(self.year()), int(self.month()), int(self.day()))

    def __str__(self):
        return str(self.to_datetime())
        
    @property
    def error(self):
        """Validate the date
        
        Return:
          - ``None`` if the date is valid else the error message
        """
        try:
            self.to_datetime()
            return None
        except ValueError, data:
            return str(data)

    def from_string(self, date):
        """Read a date from a string
        
        In:
          - ``date`` string with year, month and day (blank separator)
        """
        (year, month, day) = date.split()
        
        self.year(year)
        self.month(month)
        self.day(day)

@presentation.render_for(Date)
def render(self, h, *args):
    """Fragment to include in a ``<form>``
    """
    current_year = datetime.date.today().year
    
    return (
            h.select([h.option('%02d' % i, value=str(i)).selected(self.day()) for i in range(1, 32)]).action(self.day),
                 
            h.select([h.option(name, value=str(i+1)).selected(self.month()) for (i, name) in enumerate(Date.months)]).action(self.month),
                 
            h.select([h.option(i, value=str(i)).selected(self.year()) for i in range(current_year, current_year+5)]).action(self.year)
           )

# ---------------------------------------------------------------------------

class Table(object):
    """This component displays a table with sortable columns
    """
    def __init__(self, rows, headers, sortable_headers=(), colors=('white',), edit=None):
        """Initialization
        
        In:
          - ``rows`` -- the table data as list of lists
          - ``headers`` -- names of the columns
          - ``sortable_headers`` -- names of the sortable columns
          - ``colors`` -- list of background colors
          - ``edit`` -- callbacks function to edit
        """
        self.rows = rows
        self.headers = list(headers)
        self.sortable_headers = sortable_headers
        self.colors = colors
        self.edit = edit

        if len(sortable_headers) != 0:
            # By default, sort on the first sortable column
            self.sort_by(sortable_headers[0])
        else:
            # By default, sort on the first column
            self.sort_by(headers[0])

    def sort_by(self, name, reverse=False):
        """Criteria of the sort
        
        In:
          - ``name`` -- name of the column to sort
          - ``reverse`` -- sort in ascendant or descendant order
        """
        self.sort_header = name
        self.reverse = reverse
        i = self.headers.index(name)
        self.sort_criteria = lambda x: x[i]

    def sort(self, name):
        """Criteria of the sort
        
        In:
          - ``name`` -- name of the column to sort
        """
        reverse = False
        if self.sort_header == name:
            reverse = not self.reverse
        self.sort_by(name, reverse)

@presentation.render_for(Table)
def render(self, h, comp, *args):
    # List of the HTML for each header name
    headers = []
    for header in self.headers:
        if header in self.sortable_headers:
            # Sortable header: display its name in an active link
            headers.append(h.a(header).action(lambda name=header: self.sort(name)))
        else:
            # No sortable header: display only its name
            headers.append(header)
            
    with h.table(width='100%'):
        h << h.tr([h.th(header) for header in headers])

        # Sort and display the data
        for (i, row) in enumerate(sorted(self.rows, key=self.sort_criteria, reverse=self.reverse)):
            with h.tr(bgcolor=self.colors[i % len(self.colors)]):
                h << [h.td(column) for column in row]

                if self.edit is not None:
                    h << h.td(h.a('edit').action(lambda row=row: self.edit(comp, row)))

    return h.root

# ---------------------------------------------------------------------------

class BatchedTable(object):
    """This component displays a table with a given maximum number of rows and
    the classic ```previous`` and ``next`` actions
    """
    def __init__(self, rows, size, headers, colors=('white',)):
        """Initialization
        
        In:
          - ``rows`` -- the complete table data as list of lists
          - ``size`` - batch size (maximum number of rows displayed)
          - ``headers`` -- names of the columns
          - ``colors`` -- list of background colors
        """
        self.rows = rows
        self.offset = var.Var(0)
        self.size = size
        self.headers = list(headers)
        self.colors = colors
        
@presentation.render_for(BatchedTable)
def render(self, h, *args):
    # Dynamically create a ``Table`` component, with the subset of the data
    table = Table(
               self.rows[self.offset():self.offset()+self.size],
               self.headers, (),
               self.colors
               )
    
    # Display the table and the actions
    return (
            component.Component(table),
            h.a('<previous').action(lambda: self.offset(self.offset()-self.size)) if self.offset()-self.size >= 0 else '',
            ' | ',
            h.a('next>').action(lambda: self.offset(self.offset()+self.size)) if self.offset()+self.size < len(self.rows) else ''
           )

# ---------------------------------------------------------------------------

class SelectionTable(Table):
    """This component display a table with selectable rows and multiple actions
    
    It specializes the ``Table`` component
    """
    def __init__(self, *args, **kw):
        """Initialization
        
        In:
          - ``rows`` -- the table data as list of lists
          - ``headers`` -- names of the columns
          - ``sortable_headers`` -- names of the sortable columns
          - ``colors`` -- list of background colors
          - ``edit`` -- tuples (name to display, callback)
        """        
        super(SelectionTable, self).__init__(*args, **kw)
        self.reset_selected()   # After initialization, no row selected

    def reset_selected(self):
        """Deselect all the rows
        """
        self.selected = []
        
    def go(self, comp, f):
        """Call a function with the selected rows
        
        In:
           - ``comp`` - this component
           - ``f`` -- callback fonction to call
        """
        if len(self.selected) != 0:
            # Call the callback with all the rows and the selected ones
            f(comp, self.selected)
            self.reset_selected()
        
@presentation.render_for(SelectionTable)
def render(self, h, comp, *args):
    # List of the HTML for each header name    
    headers = []
    for header in self.headers:
        if header in self.sortable_headers:
            # Sortable header: display its name in an active link            
            headers.append(h.a(header).action(lambda name=header: self.sort(name)))
        else:
            # No sortable header: display only its name            
            headers.append(header)
    
    with h.form.pre_action(self.reset_selected):        
        with h.table:
            h << h.th()
            h << [h.th(header) for header in headers]
 
            # Sort and display the data   
            for (i, row) in enumerate(sorted(self.rows, key=self.sort_criteria, reverse=self.reverse)):
                with h.tr(bgcolor=self.colors[i % len(self.colors)]):
                    # Add a column with checkboxes
                    with h.td:
                        h << h.input(type='checkbox').action(lambda v, row=row: self.selected.append(row))
                        
                    h << [h.td(column) for column in row]
        
        # Display the action buttons
        h << [h.input(type='submit', value=name).action(lambda f=f: self.go(comp, f)) for (name, f) in self.edit]
            
        return h.root

    return h.root



# -*- coding: iso-8859-1 -*-

#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Forms with validation rules"""

from __future__ import with_statement

import datetime

from nagare import component, presentation, var, editor, validator
from nagare.examples import widgets

# --------------------------------------------------------------------------

class Form(object):
    """Object to edit"""
    def __init__(self):
        self.age = 20
        self.area = 'abcd'
        self.big = True
        self.medium = False
        self.small = True
        self.gender = 'male'
        self.color = ['blue', 'red']
        self.password = 'secret'
        self.file = None


class FormEditor(editor.Editor):
    """The form editor"""

    # The attributes of the target object, to edit
    fields = ('age', 'area', 'big', 'medium', 'small', 'gender', 'color', 'password')

    def __init__(self, target):
        """Initialization

        Create a ``editor.property`` for each ``self.fields`` of the target object

        In:
          - ``target`` -- the object to edit
        """
        super(FormEditor, self).__init__(target, self.fields)

        # Set the conversion and validation rules on the properties
        self.age.validate(lambda v: validator.to_int(v).lesser_than(50).greater_than(10).to_int())
        self.area.validate(lambda v: validator.to_string(v, strip=True).not_empty().match(r'^[a-d]+$').to_string())

        # A property can also be manually created
        self.file = editor.Property().validate(self.validate_file)

        # An editor can have more properties than the attributes of the target object
        self.confirm = editor.Property('').validate(self.validate_confirm)

    def validate_file(self, f):
        """A file is uploaded

        In:
          - ``f`` -- file object
        """
        if isinstance(f, basestring):
            return None

        return f.value

    def validate_confirm(self, v):
        """Validation of 2 related fields

        In:
          - ``v`` -- value of the ``confirm`` property

        Return:
          - ``v`` or raise an ``ValueError`` exception
        """
        if self.password() != v:
            raise ValueError('Confirm password different than password')

        return v

    def commit(self):
        """If the all properties have a valid value, write back the values
        into the target objects attributes
        """
        # Validate the ``self.fields`` properties and the ``confirm`` property
        # If they are all valid, commit the ``self.fields`` values and the
        # uploaded file content if it exits
        f = self.file.value

        if super(FormEditor, self).commit(self.fields, ('confirm',)) and f:
            self.target.file = f

    def reset_values(self):
        """Before to process the form data, the ``<option>`` and ``<checkbox>``
        properties need to be reseted
        """
        self.big(False)
        self.medium(False)
        self.small(False)

        self.color([])

# ---------------------------------------------------------------------------

# This example illustrates:
#
#  - the use of an intermediary editor object between the inputs received and
#    the target object.
#  - the association of validation methods to editor properties
#  - the highlighting of erroneous fields
#

@presentation.render_for(FormEditor)
def render(self, h, *args):
    #    print ">>>", self.age(), self.target.age, type(self.age()), type(self.target.age)

    h.head.css_url('form.css')

    # The ``pre_action`` of the ``<form>`` element is used to reset some data
    with h.form.pre_action(self.reset_values).post_action(self.commit):

        h << 'Age (between 10 and 50): ' << h.input(value=self.age()).action(self.age).error(self.age.error)
        h << h.br

        with h.fieldset:
            h << h.legend('A set of controls')

            h << 'A text (use only "[a-d]" characters):'
            h << h.textarea(self.area(), rows='2', cols='40').action(self.area).error(self.area.error)
            h << h.br << h.br

            h << 'Big' << h.input(type='checkbox').selected(self.big()).action(self.big)
            h << 'Medium' << h.input(type='checkbox').selected(self.medium()).action(self.medium)
            h << 'Small' << h.input(type='checkbox').selected(self.small()).action(self.small)
            h << h.br

            h << 'Male'
            h << h.input(type='radio', name='gender').selected(self.gender()=='male').action(lambda: self.gender('male'))
            h << ' Female'
            h << h.input(type='radio', name='gender').selected(self.gender()=='female').action(lambda: self.gender('female'))

        with h.select(multiple='multiple').action(self.color):
            with h.optgroup(label='Colors'):
                h << h.option('blue', value='blue').selected(self.color())
                h << h.option('white', value='white').selected(self.color())
            h << h.option('red', value='red').selected(self.color())
        h << h.br << h.br

        h << 'Upload a file:' << h.input(type='file').action(self.file) << h.br

        h << 'Password:' << h.input(type='password', length=10).action(self.password)
        h << ' '
        h << 'Confirm:' << h.input(type='password', length=10).action(self.confirm).error(self.confirm.error)
        h << h.hr

        h << h.input(type='submit', value='Go')

    return h.root

def example1():
    # Create a ``Form`` object, wrap it into a ``FormEditor`` and make it
    # a component
    return component.Component(FormEditor(Form()))

examples = ('Form with validation rules and "inline" error notifications', example1)

# ---------------------------------------------------------------------------

# This example illustrates:
#
#  - the use of an intermediary editor object between the inputs received and
#    the target object.
#  - the association of validation methods to editor properties
#  - the aggregation of all the errors notifications

@presentation.render_for(FormEditor, model='errors')
def render(self, h, *args):
    h.head.css_url('form.css')

    h.head.css('form-errors', '''
    .errors {
        border-top: 2px solid #c30;
        border-bottom: 2px solid #c30;
        background-color: #FFD5D5;
        margin: 8px;
    }

    .errors ol {
        margin: 4px 0 4px 0;
    }

    .errors ol li {
        font-size: 60%;
        font-weight: bold;
    }
    ''')

    errors = [(name, error) for (name, error) in (
                                                     ('Age', self.age.error),
                                                     ('Text', self.area.error),
                                                     ('Password', self.confirm.error)
                                                 ) if error]

    if len(errors):
        with h.div(class_='errors'):
            h << h.ol([h.li('%s: %s' % error) for error in errors])

    # The ``pre_action`` of the ``<form>`` element is used to reset some data
    with h.form.pre_action(self.reset_values).post_action(self.commit):
        h << 'Age (between 10 and 50): ' << h.input(value=self.age()).action(self.age)
        h << h.br

        with h.fieldset:
            h << h.legend('A set of controls')

            h << 'A text (use only "[a-d]" characters):'
            h << h.textarea(self.area(), rows='2', cols='40').action(self.area)
            h << h.br << h.br

            h << 'Big' << h.input(type='checkbox').selected(self.big()).action(self.big)
            h << 'Medium' << h.input(type='checkbox').selected(self.medium()).action(self.medium)
            h << 'Small' << h.input(type='checkbox').selected(self.small()).action(self.small)
            h << h.br

            h << 'Male'
            h << h.input(type='radio', name='gender').selected(self.gender()=='male').action(lambda: self.gender('male'))
            h << ' Female'
            h << h.input(type='radio', name='gender').selected(self.gender()=='female').action(lambda: self.gender('female'))

        with h.select(multiple='multiple').action(self.color):
            with h.optgroup(label='Colors'):
                h << h.option('blue', value='blue').selected(self.color())
                h << h.option('white', value='white').selected(self.color())
            h << h.option('red', value='red').selected(self.color())
        h << h.br << h.br

        h << 'Upload a file:' << h.input(type='file').action(self.file) << h.br

        h << 'Password:' << h.input(type='password', length=10).action(self.password)
        h << ' '
        h << 'Confirm:' << h.input(type='password', length=10).action(self.confirm)
        h << h.hr

        h << h.input(type='submit', value='Go')

    return h.root


class App:
    def __init__(self):
        self.editor = component.Component(FormEditor(Form()))

@presentation.render_for(App)
def render(self, h, *args):
    return self.editor.render(h, model='errors')

examples += ('Form with validation rules and error notifications', App)

# --------------------------------------------------------------------------

# This example illustrates the use of a predefined widget

class NameEditor(object):
    """An editor for a single text field
    """
    def __init__(self, name):
        """Initialization

        In:
          - ``name`` -- initial value of the text field
        """
        # Create a property. The text must be not empty
        self.name = editor.Property(name).validate(lambda v: validator.to_string(v, strip=True).not_empty().to_string())

    def commit(self, comp):
        """If no error, answer the text

        In:
          - ``comp`` -- the component
        """
        if not self.name.error:
            comp.answer(self.name())

@presentation.render_for(NameEditor)
def render(self, h, comp, *args):
    h.head.css_url('form.css')

    with h.form:
        h << 'Name: ' << h.input(value=self.name()).action(self.name).error(self.name.error)

        # On submit, the component answers the text
        h << h.input(type='submit', value='Change').action(lambda: self.commit(comp)) << ' '

        # On cancel, the component answers ``None``
        h << h.input(type='submit', value='Cancel').action(comp.answer)

        return h.root

def edit_name(comp, row):
    """Edit the field ``name`` of a row (field #1)

    In:
       - ``comp`` -- the current component
       - ``row`` -- a complete row

    Out:
      - ``row`` -- field ``name`` modified
    """
    # Fetch the field #1, wrap it into a ``NameEditor`` object and then make
    # it a component that replaces the current component
    name = comp.call(component.Component(NameEditor(row[1])))

    # The ``NameEditor`` answers the new field value
    # or ``None`` if the edition was canceled
    if name is not None:
        row[1] = name

def example2():
    headers = ('id', 'name')        # Name of the fields

    rows = [[2, 'a'], [3, 'b'], [1, 'c']]   # Table to display and edit

    # Create an object that display a table
    table = widgets.Table(
                          rows,                          # The data
                          headers,                       # Names of the fields
                          sortable_headers=headers,      # Names of the sortable fields
                          colors=('lightblue', 'white'), # The background of the displayed lines cycle on these colors
                          edit=edit_name                 # Callback to edit a line
                         )

    return component.Component(table)

examples += ('Sortable data grid', example2)

# --------------------------------------------------------------------------

# This example illustrates the use of a predefined widget

def example3():
    headers = ('id', 'name')        # Name of the fields

    rows = [(i, chr((i % 26)+97)*5) for i in range(1, 51)]
    table = widgets.BatchedTable(rows, 12, headers, colors=('lightblue', 'white'))

    return component.Component(table)

examples += ('Batched data grid', example3)

# --------------------------------------------------------------------------

class NamesEditor(object):
    """An editor for multiple text fields
    """
    def __init__(self, names):
        """Initialization

        Create an editor property for each names

        In:
          - ``names`` -- list of name to edit
        """
        self.names = [editor.Property(name).validate(lambda v: validator.to_string(v, strip=True).not_empty().to_string()) for name in names]

    def commit(self, comp):
        """If all the fields are valid, answer the modified fields

        In:
          - ``comp`` -- this component
        """
        if not any(name.error for name in self.names):
            comp.answer([name() for name in self.names])

@presentation.render_for(NamesEditor)
def render(self, h, comp, *args):
    h.head.css_url('form.css')

    with h.form:
        for name in self.names:
            with h.div:
                h << 'Name: '
                h << h.input(value=name()).action(name).error(name.error)

        h << h.input(type='submit', value='Changer').action(lambda: self.commit(comp))
        h << ' '
        h << h.input(type='submit', value='Annuler').action(comp.answer)

    return h.root

def modify_rows(table, selected_rows):
    """Edit the field ``name`` of the selected row

    In:
      - ``table`` -- the table widget component
      - ``selected_rows`` -- the selected rows
    """
    # Create a component to edit the field ``name`` (field #1) of the selected
    # rows, that temporary replace the table widget
    news = table.call(component.Component(NamesEditor([row[1] for row in selected_rows])))

    if news is None:
        # The modification was canceled
        return

    # Modification made, replace the ``name`` fields of the selected rows
    for (row, new) in zip(selected_rows, news):
        row[1] = new

def reset_rows(table, selected_rows):
    """Reset the field ``name`` of the selected rows

    In:
      - ``table`` -- the table widget component
      - ``selected_rows`` -- the selected rows
    """
    for row in selected_rows:
        row[1] = 'Default value'

def example4():
    rows = [[2, 'a'], [3, 'b'], [1, 'c']]
    headers = ('id', 'name')
    return widgets.SelectionTable(
                                    rows,
                                    headers, sortable_headers=headers,
                                    colors=('lightblue', 'white'),
                                    # List of the callbacks
                                    edit=(('Modifier', modify_rows), ('Reset', reset_rows))
                                   )

examples += ("Grid data with multiple actions", example4)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Actions added to navigate between the thumbnail view and the full view
of the Photos"""

from __future__ import with_statement

from nagare import presentation, component

from gallerydata import *

# ---------------------------------------------------------------------------

class Photo(object):
    def __init__(self, id):
        self.id = id

    @property
    def title(self):
        return PhotoData.get(self.id).title

    def img(self):
        return str(PhotoData.get(self.id).img)

    def thumbnail(self):
        return str(PhotoData.get(self.id).thumbnail)

@presentation.render_for(Photo)
def render(self, h, comp, *args):
    """The default view of a Photo displays the whole image"""
    
    # The image
    img = h.img.action(self.img)
    
    # On a click, the image answers ``None``
    return h.a(img).action(comp.answer)

@presentation.render_for(Photo, model='thumbnail')
def render(self, h, comp, *args):
    with h.div:
        h << h.img(width='200').action(self.thumbnail)
        h << h.br
        
        # On a click on its title, the Photo object answers itself
        h << h.a(self.title).action(lambda: comp.answer(self))
        h << h.i(' (%d octets)' % len(self.img()))

    return h.root


class Gallery(object):
    def __init__(self, name):
        self.name = name

@presentation.render_for(Gallery)
def render(self, h, comp, *args):
    with h.div:
        h << h.h1('Gallery: ', self.name)
        h << h.br

        with h.ul:
            for p in GalleryData.get_by(name=self.name).photos:
                photo = component.Component(Photo(p.id))
                
                # On a click on the title of a Photo, the Gallery temporary
                # changes itself by this Photo, displayed in its default view 
                photo.on_answer(comp.call)

                h << h.li(photo.render(h, model='thumbnail'))

    return h.root

# ---------------------------------------------------------------------------

app = lambda: Gallery(u'MyGallery')



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from elixir import *
from sqlalchemy import MetaData

__metadata__ = MetaData()

class PageData(Entity):
    pagename = Field(Unicode(40), primary_key=True)
    data = Field(Unicode(10*1024))
    creator = Field(Unicode(40))

# ---------------------------------------------------------------------------

def populate():
    page = PageData()
    page.pagename = u'FrontPage'
    page.data = u'Welcome to my *WikiWiki* !'
    page.creator = u'admin'
    
    page = PageData()
    page.pagename = u'WikiWiki'
    page.data = u'On this *WikiWiki*, the page contents can be ' \
                 'written in `Restructured Text <http://docutils.sourceforge.net/rst.html>`_'
    page.creator = u'john'



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.database import session

from sqlalchemy import Column, Integer, Unicode
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import MetaData

__metadata__ = MetaData()

# -----------------------------------------------------------------------------

Base = declarative_base(metadata=__metadata__)

class PageData(Base):
    __tablename__ = 'page'

    #id = Column(Integer, primary_key=True)
    pagename = Column(Unicode, primary_key=True)
    data = Column(Unicode(10*1024))
    creator = Column(Unicode(40))

    def __init__(self, pagename, data, creator):
        self.pagename = pagename
        self.data = data
        self.creator = creator

# -----------------------------------------------------------------------------


def populate():
    page = PageData(u'FrontPage', u'Welcome to my *WikiWiki* !', u'admin')
    database.session.add(page)

    page = PageData(u'WikiWiki', u'On this *WikiWiki*, the page contents can be '
                 'written in `Restructured Text <http://docutils.sourceforge.net/rst.html>`_',
                 u'john')
    database.session.add(page)



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""Example to demonstrate the use of asynchronous rendering of
components in a portal-like container
"""

from __future__ import with_statement

from nagare import component, presentation, ajax

from nagare.examples.wiki import wiki9
from nagare.examples.gallery import gallery7
from nagare.examples.jewels import Jewels
from nagare.examples.portal import html, tictactoe, counter, calculator
from nagare.examples.portal.source_viewer import SourceViewer

class Portal(object):
    """The ``Portal`` object is only a container"""

    def __init__(self):
        """Each components to display is simply an attribute"""
        self.calc = component.Component(calculator.Calculator())
        self.counter = component.Component(counter.Counter())
        self.tictactoe = component.Component(tictactoe.Task())
        self.wiki = component.Component(wiki9.app())
        self.jewels = component.Component(Jewels(10, 10))
        self.gallery = component.Component(gallery7.app())
        self.html = component.Component(html.Html())

@presentation.render_for(Portal)
def render(self, h, comp, *args):
    """The portal view uses YUI to display a portal-like structure with
    drag and drop aware portlets
    """
    h.head.javascript_url(ajax.YUI_INTERNAL_PREFIX + '/yahoo-dom-event/yahoo-dom-event.js')
    h.head.javascript_url(ajax.YUI_INTERNAL_PREFIX + '/dragdrop/dragdrop-min.js')

    h.head.css_url('portal.css')
    h.head.javascript_url('portal.js')

    with h.div(id='portal', class_='fixed_width'):

        # Each column must be a <div> with the class 'portlet_column'
        with h.div(class_='portlet_column'):
            # Each portlet must be a <div> with the class 'portlet'
            with h.div(class_='portlet about'):
                # Each D&D portlet handle must be a <div> with the class 'portlet_handle'
                h << h.span('Do you know?', class_='portlet_handle about_handle')

                with h.div(class_='portlet_content'):
                    h << h.p('Any Nagare component can be embedded into this portal, without any code modification!')
                    h << h.p('You can drag and drop the components')
                    h << h.p('Follow the "Click for details" link to dive into the code')

            with h.div(class_='portlet'):
                h << h.div('HTML', class_='portlet_handle')

                with h.div(class_='portlet_content'):
                    h << h.div('Add any HTML content', class_='portlet_comment')

                    # Each portlet is rendered asynchronously
                    h << h.div(self.html.render(h.AsyncRenderer()), class_='html')

                    h << h.a('Click for details', class_='source_link').action(lambda: comp.answer((self.html, html)))

            with h.div(class_='portlet'):
                h << h.div('Jewels', class_='portlet_handle')

                with h.div(class_='portlet_content'):
                    h << h.div('Click on a group of 2 or more jewels of the same color to make it disappear', class_='portlet_comment')

                    h.head.css_url('jewels.css')
                    h << self.jewels.render(h.AsyncRenderer())

            h << h.div(class_='drop_zone')

        with h.div(class_='portlet_column'):
            with h.div(class_='portlet'):
                h << h.div('TicTacToe', class_='portlet_handle')

                with h.div(class_='portlet_content'):
                    h << h.div('Enter the 2 players names then play', class_='portlet_comment')
                    h << self.tictactoe.render(h.AsyncRenderer())

                    h << h.a('Click for details', class_='source_link').action(lambda: comp.answer((self.tictactoe, tictactoe)))

            with h.div(class_='portlet'):
                h << h.div('Counter', class_='portlet_handle')
                with h.div(class_='portlet_content'):
                    h << h.div(self.counter.render(h.AsyncRenderer()), class_='comp')

                    h << h.a('Click for details', class_='source_link').action(lambda: comp.answer((self.counter, counter)))

            with h.div(class_='portlet'):
                h << h.div('Wiki', class_='portlet_handle')

                with h.div(class_='portlet_content'):
                    h << self.wiki.render(h.AsyncRenderer())

                    h << h.a('Wiki tutorial', class_='source_link', href='http://www.nagare.org/wiki')

            h << h.div(class_='drop_zone')

        with h.div(class_='portlet_column'):
            with h.div(class_='portlet'):
                h << h.div('RPN Calculator', class_='portlet_handle')

                with h.div(class_='portlet_content'):
                    with h.div(class_='portlet_comment'):
                        h << h.p('This is a RPN calculator')
                        h << h.p(u'Try: 3 \N{WHITE RIGHT-POINTING POINTER} 4 \N{MULTIPLICATION X}')

                    h << h.div(self.calc.render(h.AsyncRenderer()), class_='comp')

                    h << h.a('Click for details', class_='source_link').action(lambda: comp.answer((self.calc, calculator)))

            with h.div(class_='portlet'):
                h << h.div('Photos gallery', class_='portlet_handle')

                with h.div(class_='portlet_content'):
                    h << h.div('Upload pictures', class_='portlet_comment')
                    h << self.gallery.render(h.AsyncRenderer())
                    h << h.p(style='clear: both')

            h << h.div(class_='drop_zone')

    # Initialize the portal
    h << h.script('init_portal()', type='text/javascript')

    return h.root


class App(object):
    def __init__(self):
        self.portal = component.Component(Portal())
        self.source = component.Component(None)

        self.portal.on_answer(lambda args: self.source.call(SourceViewer(*args)))

@presentation.render_for(App)
def render(self, h, comp, *args):
    h.head << h.head.title('Demonstration portal')
    h.head.css_url('/static/nagare/application.css')

    with h.body:
        if self.source():
            h << { 'style' : 'overflow: hidden' }
            h << h.div(id='overlay1')
            h << h.div(self.source, id='overlay2')

        with h.div(id='body'):
            h << h.a(h.img(src='/static/nagare/img/logo_small.png'), id='logo', href='http://www.nagare.org/', title='Nagare home')

            with h.div(id='content'):
                h << h.div('Demonstrations portal', id='title')

                h << h.div(self.portal, id='main')

    return h.root

# ---------------------------------------------------------------------------

app = App



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from __future__ import with_statement

import inspect

from pygments import highlight, lexers, formatters

from nagare import presentation

class HtmlFormatter(formatters.HtmlFormatter):
    def __init__(self, hl_lines, *args, **kw):
        super(HtmlFormatter, self).__init__(*args, **kw)
        self.marker_lines = {}
        self.hl_lines = {}

        for (i, (marker_lines, _, lines)) in enumerate(hl_lines):
            for marker_line in marker_lines:
                self.marker_lines[marker_line-1] = i

            for line in lines:
                self.hl_lines.setdefault(line-1, []).append(i)

    def wrap(self, source, outfile):
        return self._wrap(source)

    def _wrap(self, source):
        for (i, (code, line)) in enumerate(source):
            if code:
                markers = self.hl_lines.get(i)
                if markers is not None:
                    line = '<span class="%s">%s</span>' % (' '.join('block_%d' % marker for marker in markers), line)

                marker = self.marker_lines.get(i)
                if marker is not None:
                    line = '<span class="highlight_block_marker" name="%d">%s</span>' % (marker, line)

            yield (code, line)


class SourceViewer(object):
    def __init__(self, comp, mod):
        self.code = '\n'.join(line for i, line in enumerate(inspect.getsource(mod).splitlines()) if i+1 in mod.hl_lines[0])
        self.comp = comp
        self.hl_lines = mod.hl_lines[1:]

@presentation.render_for(SourceViewer)
def render(self, h, comp, *args):
    h.head.css_url('/static/nagare/application.css')
    h.head.css_url('portal.css')

    with h.div(id='source_viewer'):
        with h.div(class_='left'):
            h << h.a(u'\u2715', href='#', id='popup_close').action(comp.answer)

            with h.p(id='help'):
                h << 'Hold your mouse over the '
                h << h.img(src='attention_small.png')
                h << ' icons to follow the tutorial'

            with h.div(class_='comp'):
                h << self.comp.render(h.AsyncRenderer())

            for (i, (_, description, _)) in enumerate(self.hl_lines):
                h << h.div(h.parse_htmlstring(description), class_='description', id='description_%d' % i, style='display: none')

        with h.div(class_='right'):
            code = highlight(self.code, lexers.PythonLexer(), HtmlFormatter(self.hl_lines, nowrap=False, noclasses=True, linenos='inline'))
            h << h.pre(h.parse_htmlstring(code))


        h << h.script('init_sourceviewer()')

    return h.root



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

VERSION='0.3.0'

from setuptools import setup, find_packages

setup(
      name = 'nagare.examples',
      version = VERSION,
      author = 'Alain Poirier',
      author_email = 'alain.poirier at net-ng.com',
      description = 'Demo and examples for the Nagare web framework',
      long_description = '''Read the `installation document <http://www.nagare.org/trac/wiki/DemoInstallation>`_
      to do a standard installation or a developer installation with the
      `latest <http://www.nagare.org/snapshots/nagare.examples-latest#egg=nagare.examples-dev>`_
      development version from the `mercurial repository <http://hg.nagare.org/examples>`_.
      ''',
      license = 'BSD',
      keywords = 'web wsgi framework sqlalchemy elixir seaside continuation ajax stackless',
      url = 'http://www.nagare.org',
      download_url = 'http://www.nagare.org/download',
      packages = find_packages(),
      include_package_data = True,
      package_data = {'' : ['*.cfg']},
      use_hg_version = True,
      zip_safe = False,
      dependency_links = ('http://www.nagare.org/download/',),
      install_requires = ('nagare[database,doc]>=0.4.0', 'PIL'),
      namespace_packages = ('nagare', 'nagare.examples',),
      entry_points = '''
      [nagare.applications]
      demo = nagare.examples.demo:app
      wiki = nagare.examples.wiki.wiki9:app
      gallery = nagare.examples.gallery.gallery7:app
      portal = nagare.examples.portal.portal:app
      jewels = nagare.examples.jewels:app
      chat = nagare.examples.chat:app
      ''',
      classifiers = (
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Environment :: Web Environment',
        'Operating System :: Microsoft :: Windows :: Windows NT/2000',
        'Operating System :: Unix',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
      )
     )



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.database import session

from sqlalchemy import Table, Column, Unicode
from sqlalchemy import MetaData

__metadata__ = MetaData()

# -----------------------------------------------------------------------------

page_data = Table(
                    'page', __metadata__,
                    Column('pagename', Unicode, primary_key=True),
                    Column('data', Unicode(10*1024)),
                    Column('creator', Unicode(40))
                 )




#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

from nagare.database import session

from sqlalchemy import MetaData, Table, Column, Unicode, BLOB, ForeignKey, Integer
from sqlalchemy.orm import mapper, relation, deferred

from nagare.examples.gallery.gallery8 import Photo, Gallery

__metadata__ = MetaData()

photo_table = Table(
                    'photo', __metadata__,
                    Column('title', Unicode(100), primary_key=True),
                    Column('img', BLOB),
                    Column('thumbnail', BLOB),
                    Column('size', Integer),

                    Column('gallery_id', Unicode(40), ForeignKey('gallery.name'))
                   )

gallery_table = Table(
                     'gallery', __metadata__,
                     Column('name', Unicode(40), primary_key=True)
                    )

mapper(Photo, photo_table, properties={ 'img' : deferred(photo_table.c.img) })
mapper(Gallery, gallery_table, properties={ 'photos' : relation(Photo, backref='gallery') })

# ---------------------------------------------------------------------------

def populate():
    database.session.add(Gallery(name=u'MyGallery'))



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""More complex security rules: only the author can change a page content

'guest / guest' is an editor but can't edit the 'WikiWiki' page
'john / john' is an editor and the creator of the 'WikiWiki' page so he
can edit it
Then try as the administrator 'admin / admin' that can edit all the pages
"""

from __future__ import with_statement

import re
import docutils.core

from nagare import component, presentation, var, security, wsgi, log

from wikidata import PageData

# ---------------------------------------------------------------------------

class Login:
    pass

@presentation.render_for(Login)
def render(self, h, binding, *args):
    user = security.get_user()

    if not user:
        html = h.form(
                      'Login: ', h.input(name='__ac_name'), ' ',
                      'Password: ', h.input(type='password', name='__ac_password'), ' ',
                      h.input(type='submit', value='ok')
                     )
    else:
        html = (
                'Welcome ', h.b(user.id), h.br,
                h.a('logout').action(lambda: security.get_manager().logout())
               )

    return html

# ---------------------------------------------------------------------------

wikiwords = re.compile(r'\b([A-Z]\w+[A-Z]+\w+)')

class Page(object):
    def __init__(self, title):
        self.title = title

    def edit(self, comp):
        content = comp.call(PageEditor(self))

        if content is not None:
            log.info('New content for page [%s]: [%s...]' % (self.title, content[:50]))
            security.check_permissions('wiki.editor', self)
            page = PageData.get_by(pagename=self.title)
            page.data = content

            # If the creator of the page is not already set, set it with the
            # current user
            if page.creator is None:
                page.creator = security.get_user().id
                log.debug('New creator for page [%s]: [%s]' % (page.pagename, page.creator))

@presentation.render_for(Page)
def render(self, h, comp, *args):
    page = PageData.get_by(pagename=self.title)

    content = docutils.core.publish_parts(page.data, writer_name='html')['html_body']
    content = wikiwords.sub(r'<wiki>\1</wiki>', content)
    html = h.parse_htmlstring(content, fragment=True)[0]

    for node in html.getiterator():
        if node.tag == 'wiki':
            a = h.a(node.text, href='page/'+node.text).action(lambda title=unicode(node.text): comp.answer(title))
            node.replace(a)

    h << html

    if security.has_permissions('wiki.editor', self):
        h << h.a('Edit this page', href='page/'+self.title).action(lambda: self.edit(comp))

    return h.root

# The meta data view now also displays the creator's name
@presentation.render_for(Page, model='meta')
def render(self, h, comp, *args):
    h << h.span('Viewing ') << h.b(self.title) << h.br

    page = PageData.get_by(pagename=self.title)
    if page.creator:
        h << h.i('Created by ', page.creator) << h.br

    h << 'You can return to the ' << h.a('FrontPage', href='page/FrontPage')

    return h.root

# ---------------------------------------------------------------------------

class PageEditor(object):
    def __init__(self, page):
        self.page = page

@presentation.render_for(PageEditor)
def render(self, h, comp, *args):
    content = var.Var()

    page = PageData.get_by(pagename=self.page.title)

    with h.form:
        with h.textarea(rows='10', cols='40').action(content):
            h << page.data
        h << h.br
        h << h.input(type='submit', value='Save').action(lambda: comp.answer(content()))
        h << ' '
        h << h.input(type='submit', value='Cancel').action(comp.answer)

    return h.root

@presentation.render_for(PageEditor, model='meta')
def render(self, h, *args):
    return ('Editing ', h.b(self.page.title))

# ---------------------------------------------------------------------------

class Wiki(object):
    def __init__(self):
        self.login = component.Component(Login())
        self.content = component.Component(None)
        self.content.on_answer(self.goto)

        self.goto(u'FrontPage')

    def goto(self, title):
        page = PageData.get_by(pagename=title)
        if page is None:
            log.info('New wiki page: [%s]' % title)
            PageData(pagename=title, data='')

        self.content.becomes(Page(title))

@presentation.render_for(Wiki)
def render(self, h, comp, *args):
    h.head.css('main_css', '''
    .document:first-letter { font-size:2em }
    .meta { float:right; width: 10em; border: 1px dashed gray;padding: 1em; margin: 1em; }
    .login { font-size:0.75em; }
    ''')

    with h.div(class_='login'):
        h << self.login

    with h.div(class_='meta'):
        h << self.content.render(h, model='meta')

    h << self.content << h.hr

    if security.has_permissions('wiki.admin', self):
        h << 'View the ' << h.a('complete list of pages', href='all').action(lambda: self.goto(comp.call(self, model='all')))

    return h.root

@presentation.render_for(Wiki, model='all')
@security.permissions('wiki.admin')
def render(self, h, comp, *args):
    with h.ul:
        for page in PageData.query.order_by(PageData.pagename):
            with h.li:
                h << h.a(page.pagename, href='page/'+page.pagename).action(lambda title=page.pagename: comp.answer(title))

    return h.root

# ---------------------------------------------------------------------------

@presentation.init_for(Wiki, "(len(url) == 2) and (url[0] == 'page')")
def init(self, url, *args):
    title = url[1]

    page = PageData.get_by(pagename=title)
    if page is None:
        raise presentation.HTTPNotFound()

    self.goto(title)

@presentation.init_for(Wiki, "len(url) and (url[0] == 'all')")
def init(self, url, comp, *args):
    component.call_wrapper(lambda: self.goto(comp.call(self, model='all')))

# ---------------------------------------------------------------------------

from peak.rules import when
from nagare.security import common

def flatten(*args):
    return sum([flatten(*x) if hasattr(x, '__iter__') else (x,) for x in args], ())

class User(common.User):
    def __init__(self, id, roles=()):
        super(User, self).__init__(id)
        self.roles = flatten(roles)

    def has_permission(self, permission):
        return permission in self.roles

editor_role = ('wiki.editor',)
admin_role = ('wiki.admin', editor_role)

from nagare.security import form_auth

class Authentication(form_auth.Authentication):
    def get_password(self, username):
        return username

    def _create_user(self, username):
        if username is None:
            return None

        if (username == 'john') or (username == 'guest'):
            return User(username, editor_role)

        if username == 'admin':
            return User(username, admin_role)

        return User(username)


class Rules(common.Rules):
    # A administrator has all the permissions on the Wiki
    @when(common.Rules.has_permission, (User, str, Wiki))
    def _(next_method, self, user, perm, subject):
        return user.has_permission('wiki.admin') or next_method(self, user, perm, subject)

    # 1. An administrator has all the permissions on the pages
    #
    # 2. A user can modify a page if he has the 'wiki.editor' permission
    #    and if he is the creator of the page, or if the page has no creator
    #    yet (page just created)
    @when(common.Rules.has_permission, (User, str, Page))
    def _(self, user, perm, subject):
        if user.has_permission('wiki.admin'):
            return True

        creator = PageData.get_by(pagename=subject.title).creator
        if user.has_permission(perm) and (perm != 'wiki.editor' or (creator is None) or (creator == user.id)):
            return True

        return common.Denial()


class SecurityManager(Authentication, Rules):
    pass

# ---------------------------------------------------------------------------

class WSGIApp(wsgi.WSGIApp):
    def __init__(self, app_factory):
        super(WSGIApp, self).__init__(app_factory)
        self.security = SecurityManager()

app = WSGIApp(lambda: component.Component(Wiki()))



#--
# Copyright (c) 2008-2012 Net-ng.
# All rights reserved.
#
# This software is licensed under the BSD License, as described in
# the file LICENSE.txt, which you should have received as part of
# this distribution.
#--

"""A Wiki component is added that manage the navigation into the pages
"""

from __future__ import with_statement

import re
import docutils.core

from nagare import component, presentation, var

from wikidata import PageData

# ---------------------------------------------------------------------------

wikiwords = re.compile(r'\b([A-Z]\w+[A-Z]+\w+)')

class Page(object):
    def __init__(self, title):
        self.title = title

    def edit(self, comp):
        content = comp.call(PageEditor(self))
        
        if content is not None:
            page = PageData.get_by(pagename=self.title)
            page.data = content
        
@presentation.render_for(Page)
def render(self, h, comp, *args):
    page = PageData.get_by(pagename=self.title)

    content = docutils.core.publish_parts(page.data, writer_name='html')['html_body']
    content = wikiwords.sub(r'<wiki>\1</wiki>', content)
    html = h.parse_htmlstring(content, fragment=True)[0]

    for node in html.getiterator():
        if node.tag == 'wiki':
            a = h.a(node.text).action(lambda title=unicode(node.text): comp.answer(title))
            node.replace(a)

    return (html, h.a('Edit this page').action(lambda: self.edit(comp)))

# ---------------------------------------------------------------------------

class PageEditor(object):
    def __init__(self, page):
        self.page = page
        
@presentation.render_for(PageEditor)
def render(self, h, comp, *args):
    content = var.Var()

    page = PageData.get_by(pagename=self.page.title)
    
    with h.form:
        with h.textarea(rows='10', cols='40').action(content):
            h << page.data
        h << h.br
        h << h.input(type='submit', value='Save').action(lambda: comp.answer(content()))
        h << ' '
        h << h.input(type='submit', value='Cancel').action(comp.answer)
                                                             
    return h.root

# ---------------------------------------------------------------------------

class Wiki(object):
    """Wiki component. Into the components graph, it'sthe parent of the
    displayed pages.
    """
    def __init__(self):
        # The ``content`` contains the page component currently displayed
        self.content = component.Component(None)
        # A page answers the WikiWord clicked. Bind the answer to the ``goto()``
        # method
        self.content.on_answer(self.goto)
        
        # The first displayed page is the ``FrontPage`` page
        self.goto(u'FrontPage')

    def goto(self, title):
        """Navigate to an other page
        
        In:
          - ``title`` -- title of the new page to display
        """
        # Retrieve the new page data from the database
        page = PageData.get_by(pagename=title)
        if page is None:
            # The new page doesn't exist, create it in database
            PageData(pagename=title, data='')
        
        # Permanently replace, into the component graph, the currently
        # displayed page by the new one
        self.content.becomes(Page(title))
        
@presentation.render_for(Wiki)
def render(self, h, *args):
    h.head.css('main_css', '''
    .document:first-letter { font-size:2em }
    ''')
    
    return self.content

# ---------------------------------------------------------------------------

app = Wiki
